#!/usr/bin/python

#-----------------------------------------------------------------#
# A python code for parsing RS2 XML file into python data structures
# and from there into DORIS res file structure
#
# Author: TUDelft - 2010
# Maintainer: Mahmut Arikan
#
# Developed based on tsx_dump_header2doris.py.
#
# this is rather fair implementation and should be used as a structure
# for the future implementation of XML/GeoTIFF data readers

# NO XPATH version
# 2012.Nov  Fix for ascending mode time parameters Samie Esfahany and Mahmut
# 2013.Sep  Fix for UF and MF acquisition modes and cleanup, Piers van der TOrren
#-----------------------------------------------------------------#

from lxml import etree
import sys
from datetime import datetime

#try:
#    from lxml import etree
#except ImportError:
#    import xml.etree.ElementTree as etree


codeRevision=1.2   # this code revision number

def usage():
    print 'INFO    : @(#)Doris InSAR software, $Revision: %s $, $Author: TUDelft $' % codeRevision
    print
    print 'Usage   : python rs2_dump_header2doris.py rs2_XML_product > outputfile'
    print '                           where rs2_XML_product is the input filename'
    print
    print '          This software is part of Doris InSAR software package.\n'
    print '(c) 1999-2010 Delft University of Technology, the Netherlands.\n'

try:
    inputFileName  = sys.argv[1]
#    outputFileName = sys.argv[2]
#    outStream      = open(outputFileName,'w')
except:
    print '\nError   : Unrecognized input or missing arguments\n\n'
    usage()
    sys.exit(1)


# Helper functions
def nsmap_none(path, ns='None:'):
    """ Add a namespace to each tag in the given path which doesn't have one.
    """
    def add_ns_if_none(tag):
        if tag in ('', '.', '*') or ':' in tag:
            return tag
        else:
            return ''.join((ns, tag))
    return '/'.join(add_ns_if_none(tag) for tag in path.split('/'))

def hms2sec(hmsString):
    """ input hmsString syntax: XX:XX:XX.xxxxxx
    """
    return int(hmsString[0:2])*3600 + \
        int(hmsString[3:5])*60 + \
        float(hmsString[6:])

# constants
SOL = 299792458.0    # speed of light
dateformat = '%Y-%m-%dT%H:%M:%S.%fZ'

# default namespace
nsmap = {None:"http://www.rsi.ca/rs2/prod/xml/schemas"}
ns = '{' + nsmap[None] + '}'


inTree = etree.parse(inputFileName)

# query syntax for every field
queryList = {
             # mission info
             'mission'             : 'sourceAttributes/satellite',
             # imageFile file
             # fullResolutionImageData pole="HH" # inTree.findall('imageAttributes/fullResolutionImageData')[0].text, and more [1] .. [3]
             'imageFile'           : 'imageAttributes/fullResolutionImageData',
             #'imageLines'          : 'imageAttributes/rasterAttributes/numberOfLines',
             'imageLines'          : 'imageAttributes//numberOfLines',
             'imagePixels'         : 'imageAttributes//numberOfSamplesPerLine',
             'imageLineSpacing'    : 'imageAttributes//sampledLineSpacing',
             'imagePixelSpacing'   : 'imageAttributes//sampledPixelSpacing',
             # volume info
             #'volFile' : 'productComponents/annotation/file/location/filename', # HARDCODED!!! for radarsat-2
             # following smt like Level 1B Product, check manual
             'volID'               : 'productId',
             'volRef'              : 'documentIdentifier',
             # product info
             #productSpec'          : 'generalHeader/referenceDocument',  # TSX
             'productSpec'         : 'documentIdentifier',
             'productVolDate'      : 'imageGenerationParameters//processingTime',
             'productSoftVer'      : 'imageGenerationParameters//softwareVersion',
             'productDate'         : 'sourceAttributes/rawDataStartTime',
             'productFacility'     : 'imageGenerationParameters//processingFacility',
             # scene info
             #'scenePol'           : 'sourceAttributes/radarParameters/acquisitionType',    # Fine Quad Polarization
             'scenePol'            : 'sourceAttributes//polarizations',
             'sceneBeam'           : 'sourceAttributes//beams',
             'sceneBeamMode'       : 'sourceAttributes/beamModeMnemonic',
             'list_sceneLat'       : 'imageAttributes/geographicInformation/geolocationGrid/imageTiePoint/geodeticCoordinate/latitude',
             'list_sceneLon'       : 'imageAttributes/geographicInformation/geolocationGrid/imageTiePoint/geodeticCoordinate/longitude',
             'sceneRecords'        : 'imageGenerationParameters/sarProcessingInformation/numberOfLinesProcessed',
             'antennaLookDir'      : 'sourceAttributes//antennaPointing',
             'missinglines'        : 'sourceAttributes//numberOfMissingLines',
             # orbit info
             'orbitABS'            : 'sourceAttributes/orbitAndAttitude//orbitDataFile',
             'orbitDir'            : 'sourceAttributes//passDirection',
             'list_orbitTime'      : 'sourceAttributes//stateVector/timeStamp',
             'list_orbitX'         : 'sourceAttributes//stateVector/xPosition',
             'list_orbitY'         : 'sourceAttributes//stateVector/yPosition',
             'list_orbitZ'         : 'sourceAttributes//stateVector/zPosition',
             'list_orbitXV'        : 'sourceAttributes//stateVector/xVelocity',
             'list_orbitYV'        : 'sourceAttributes//stateVector/yVelocity',
             'list_orbitZV'        : 'sourceAttributes//stateVector/zVelocity',
             # range
             'list_rangeRSR'       : 'sourceAttributes//adcSamplingRate', # for UF mode there are two subpulses which have to be added together
             'rangeBW'             : 'imageGenerationParameters//rangeLookBandwidth',
             'rangeWind'           : 'imageGenerationParameters//rangeWindow/windowName',
             'rangeWindCoeff'      : 'imageGenerationParameters//rangeWindow/windowCoefficient',
             'rangeTimePix'        : 'imageGenerationParameters//slantRangeTimeToFirstRangeSample',
             # azimuth
             'azimuthPRF'          : 'sourceAttributes//pulseRepetitionFrequency', # for some modes (MF, UF) this value is changed in processing, calculate from other values
             'azimuthBW'           : 'imageGenerationParameters//azimuthLookBandwidth',
             'azimuthWind'         : 'imageGenerationParameters//azimuthWindow/windowName',
             'azimuthWindCoeff'    : 'imageGenerationParameters//azimuthWindow/windowCoefficient',
             'azimuthTimeFirstLine': 'imageGenerationParameters//zeroDopplerTimeFirstLine',
             'azimuthTimeLastLine' : 'imageGenerationParameters//zeroDopplerTimeLastLine',
             # doppler
             'dopplerTime'         : 'imageGenerationParameters//timeOfDopplerCentroidEstimate',
             'dopplerCoeff'        : 'imageGenerationParameters//dopplerCentroidCoefficients',
             # for wavelength computation
             'radarfreq'           : 'sourceAttributes//radarCenterFrequency',
             #  wavelength_computed = (0.000000001*SOL/atof(c8freq)) seems more reliable, BK 03/04
             }


# get variables and parameters from xml
container = {}
for key, value in queryList.iteritems():
    if key.startswith('list_'):
        container[key] = [tag.text for tag in inTree.findall(nsmap_none(value, ns))]
    else:
        container[key] = inTree.findtext(nsmap_none(value, ns))
        if container[key] == None:
            raise Exception('Path {0} not found in XML'.format(value))

container['dopplerCoeff'] = container['dopplerCoeff'].split()

def mean(l):
    return sum(l)/len(l)
container['sceneCenLat'] = mean([float(val) for val in container['list_sceneLat']])
container['sceneCenLon'] = mean([float(val) for val in container['list_sceneLon']])

# sum subpulses for UF like modes
RSR1 = sum(float(val) for val in container['list_rangeRSR'])
# alternative: calculate RSR from given pixel spacing, former way doesn't work correctly for reduced resolution XF images. Difference is a factor 1.0000001
container['rangeRSR'] = SOL/float(container['imagePixelSpacing'])/2
# for backwards compatibility use first method when values are very close to each other
if 0.9999 < RSR1/container['rangeRSR'] < 1.0001:
    container['rangeRSR'] = RSR1

# Calculate PRF
azimuthTimeFirstLine = datetime.strptime(container['azimuthTimeFirstLine'], dateformat)
azimuthTimeLastLine = datetime.strptime(container['azimuthTimeLastLine'], dateformat)
obs_time = (azimuthTimeLastLine - azimuthTimeFirstLine).total_seconds()
# set start time to the first observed line (for ascending the image is flipped)
if obs_time > 0:
    azimuthTimeStart = azimuthTimeFirstLine
else:
    azimuthTimeStart = azimuthTimeLastLine
    obs_time = -obs_time

if container['sceneBeam'] != 'S3': # Hacky fix for S3 merged images
    container['azimuthPRF'] = (float(container['imageLines']) - 1)/obs_time

# ---------------------------------------------------------------------------------------------------------

#print container['mission']
#exit()

dummyVar = 'DUMMY'

print('\nrs2_dump_header2doris.py v%s, doris software, 2013\n' % codeRevision)
print('*******************************************************************')
print('*_Start_readfiles:')
print('*******************************************************************')
print('Volume file: 					%s' % 'product.xml') # container['volFile']) # HARDCODED!!! for Radarsat-2
print('Volume_ID: 					%s' % container['volID'])
print('Volume_identifier: 				%s' % container['volRef'])
print('Volume_set_identifier: 				%s' % dummyVar)
print('(Check)Number of records in ref. file: 		%s' % container['sceneRecords'])
print('SAR_PROCESSOR:                                  %s %s' % (str.split(container['productSpec'])[0][:2],container['productSoftVer']))
print('SWATH:                                          %s' % container['sceneBeam'])
print('PASS:                                           %s' % container['orbitDir'])
print('IMAGING_MODE:                                   %s %s' % (container['sceneBeamMode'],container['scenePol']))
print('RADAR_FREQUENCY (Hz):                           %s' % container['radarfreq'])
print('')
print('Product type specifier: 	                %s' % container['mission'])
print('Logical volume generating facility: 		%s' % container['productFacility'])
print('Logical volume creation date: 			%s' % container['productVolDate'])
print('Location and date/time of product creation: 	%s' % container['productDate'])
#print('Scene identification: 				Orbit: %s %s Mode: %s' % (container['orbitABS'].split('_')[0],container['orbitDir'],container['sceneBeamMode']))
print('Scene identification: 				Orbit: %s  %s' % (container['orbitABS'].split('_')[0], azimuthTimeStart.strftime(dateformat)))
print('Scene location: 		                lat: %.4f lon: %.4f' % (float(container['sceneCenLat']),float(container['sceneCenLon'])))
print('')
print('Leader file:                                 	%s' % 'product.xml') # container['volFile']) # HARDCODED!!! for Radarsat-2
print('Sensor platform mission identifer:         	%s' % container['mission'])
print('Scene_centre_latitude:                     	%s' % container['sceneCenLat'])
print('Scene_centre_longitude:                    	%s' % container['sceneCenLon'])
print('Scene_centre_heading:                            %s' % 'Null') # needs to be computed from geoinfo
print('Radar_wavelength (m):                      	%s' % str(SOL/float(container['radarfreq'])))
print('First_pixel_azimuth_time (UTC):			%s' % azimuthTimeStart.strftime('%d-%b-%Y %H:%M:%S.%f'))
print('Pulse_Repetition_Frequency (computed, Hz): 	%s' % container['azimuthPRF'])
print('Total_azimuth_band_width (Hz):             	%s' % float(container['azimuthBW']))
print('Weighting_azimuth:                         	%s %f' % (str.upper(container['azimuthWind']), float(container['azimuthWindCoeff'])))
print('Xtrack_f_DC_constant (Hz, early edge):     	%s' % container['dopplerCoeff'][0])
print('Xtrack_f_DC_linear (Hz/s, early edge):     	%s' % container['dopplerCoeff'][1])
print('Xtrack_f_DC_quadratic (Hz/s/s, early edge): 	%s' % container['dopplerCoeff'][2])
print('Range_time_to_first_pixel (2way) (ms):     	%0.15f' % (float(container['rangeTimePix'])*1000))
print('Range_sampling_rate (computed, MHz):       	%0.6f' % (float(container['rangeRSR'])/1000000))
print('Total_range_band_width (MHz):               	%s' % (float(container['rangeBW'])/1000000))
print('Weighting_range:                            	%s %f' % (str.upper(container['rangeWind']), float(container['rangeWindCoeff'])))
print('')
print('*******************************************************************')
print('Datafile: 					%s' % container['imageFile'])
print('Dataformat: 				%s' % 'GeoTIFF')  # hardcoded!!!
print('Number_of_lines_original: 			%s' % container['imageLines'])
print('Number_of_pixels_original: 	                %s' % container['imagePixels'])
print('*******************************************************************')
print('* End_readfiles:_NORMAL')
print('*******************************************************************')
print('')
print('')
print('*******************************************************************')
print('*_Start_leader_datapoints:  %s ' % container['orbitABS'].split('_')[1])
print('*******************************************************************')
print(' t(s)		X(m)		Y(m)		Z(m)      X_V(m/s)      Y_V(m/s)      Z_V(m/s)')
print('NUMBER_OF_DATAPOINTS: %s' % len(container['list_orbitTime']))

# MA : positions and velocities
for i in range(len(container['list_orbitTime'])):
    print(' %.6f %s %s %s %s %s %s' % (hms2sec(container['list_orbitTime'][i].split('T')[1].strip('Z')),
                                  container['list_orbitX'][i],
                                  container['list_orbitY'][i],
                                  container['list_orbitZ'][i],
                                  container['list_orbitXV'][i],
                                  container['list_orbitYV'][i],
                                  container['list_orbitZV'][i]))

print('')
print('*******************************************************************')
print('* End_leader_datapoints:_NORMAL')
print('*******************************************************************')
# print('\n')
# print('    Current time: %s\n' % time.asctime())
# print('\n')

#EOF



# This file contains a function to check which files for sentinel are available, which ones are downloaded and a quality
# check for the files which are downloaded.

import urllib
import urllib2
import ssl
import re
import os, sys
import datetime
import base64
import subprocess
from fiona import collection
from fastkml import kml
from lxml import etree
import xml.etree.ElementTree as ET


def sentinel_available(start_day='', end_day='', sensor_mode='', product='', level='', track='', polarisation='', orbit_direction='', ROI='', user='', password=''):
    # All available sentinel 1 images are detected and printed on screen.
    # Following variables can be used to make a selection.
    # shape > defining shape file or .kml
    # start_day > first day for downloads (default one month before now) [yyyymmdd]
    # end_day > last day for downloads (default today)
    # track > the tracks we want to check (default all)
    # polarisation > which polarisation will be used. (default all)

    # string is the field we enter as url
    string = ''

    if sensor_mode:
        string = string + ' AND ' + 'sensoroperationalmode:' + sensor_mode
    if product:
        string = string + ' AND ' + 'producttype:' + product
    if level:
        string = string + ' AND ' + level
    if orbit_direction:
        string = string + ' AND ' + 'orbitdirection:' + orbit_direction
    if track:
        string = string + ' AND ' + 'relativeorbitnumber:' + track
    if start_day:
        start = datetime.datetime.strptime(start_day, '%Y-%m-%d').strftime('%Y-%m-%d')
    else:
        start = (datetime.datetime.now() - datetime.timedelta(days=350)).strftime('%Y-%m-%d')
    if end_day:
        end = datetime.datetime.strptime(end_day, '%Y-%m-%d').strftime('%Y-%m-%d')
    else:
        end = datetime.datetime.now().strftime('%Y-%m-%d')
    if polarisation:
        string = string + ' AND ' + 'polarisationmode:' + polarisation
    if ROI:
        shape_str = load_shape_info(ROI)
        string = string + ' AND footprint:"Intersects(POLYGON(' + shape_str + '))"'

    date_string = 'beginPosition:[' + start + 'T00:00:00.000Z TO ' + end + 'T23:59:59.999Z] AND endPosition:[' + start + 'T00:00:00.000Z TO ' + end + 'T23:59:59.999Z]'
    string = string + ' AND ' + date_string

    # Finally we do the query to get the search result.
    string = string[5:] + '&rows=1000'
    url = 'https://scihub.copernicus.eu/dhus/search?q=' + urllib.quote_plus(string)
    print(url)

    print('Requesting available products: ' + url)
    request = urllib2.Request(url)
    base64string = base64.b64encode('%s:%s' % (user, password))
    request.add_header("Authorization", "Basic %s" % base64string)

    # connect to server. Hopefully this works at once
    try:
        dat = urllib2.urlopen(request)
    except:
        print 'not possible to connect this time'
        return [], [], []

    html_dat = ''
    for line in dat:
        html_dat = html_dat + line

    parser = etree.HTMLParser()
    tree = etree.fromstring(html_dat, parser)
    products = [data for data in tree.iter(tag='entry')]
    links = [data.find('link').attrib for data in tree.iter(tag='entry')]
    dates = [data.findall('date')[1].text for data in tree.iter(tag='entry')]

    print('Following products will be downloaded')
    for link in links:
        print(link)


    return products, links, dates


def load_shape_info(shapefile):
    # This script converts .kml, .shp and .txt files to the right format. If multiple shapes are available the script
    # will select the first one.

    if shapefile.endswith('.shp'):
        with collection(shapefile, "r") as inputshape:
            for shape in inputshape:
                # only first shape
                dat = shape['geometry']['coordinates']

                st='('
                for p in dat[0]:
                    st = st + str(p[0]) + ' ' + str(p[1]) + ','
                st = st[:-1] + ')'

                break
    elif shapefile.endswith('.kml'):
        doc = file(shapefile).read()
        k = kml.KML()
        k.from_string(doc)
        shape = list(list(k.features())[0].features())[0].geometry.exterior.coords[:]
        st='('
        for p in shape:
            st = st + str(p[0]) + ' ' + str(p[1]) + ','
        st = st[:-1] + ')'
    else:
        print 'format not recognized! Pleas creat either a .kml or .shp file.'
        return []

    return st


def sentinel_check_validity(products=[], destination_folder='', user='', password='', remove=True):
    # Check if the downloaded files are valid and remove if not

    valid_files = []
    invalid_files = []

    if not products:
        print 'Nothing to check'
        return

    for product in products:
        date = str(product.findall('date')[1].text)
        date = datetime.datetime.strptime(date[:19], '%Y-%m-%dT%H:%M:%S')

        name = str(product.find('title').text)

        track = str(product.find('int[@name="relativeorbitnumber"]').text)
        data_type = str(product.find(".//str[@name='filename']").text)[4:16]
        pol = str(product.find(".//str[@name='polarisationmode']").text).replace(' ', '')
        direction = str(product.find(".//str[@name='orbitdirection']").text)
        if direction == 'ASCENDING':
            direction = 'asc'
        elif direction == 'DESCENDING':
            direction = 'dsc'

        trackfolder = os.path.join(destination_folder, 's1_' + direction + '_t' + track)
        typefolder = os.path.join(trackfolder, data_type + '_' + pol)
        datefolder = os.path.join(typefolder, date.strftime('%Y%m%d'))

        xml_dir = os.path.join(datefolder, name + '.xml')
        file_dir = os.path.join(datefolder, name + '.SAFE.zip')
        kml_dir = os.path.join(datefolder, name + '.kml')
        preview_dir = os.path.join(datefolder, name + '.jpg')

        # First check the file
        if os.path.exists(file_dir):
            uuid = product.find('id').text
            valid_dat = sentinel_quality_check(file_dir, uuid, user, password)
        else:
            valid_dat = False

        if not valid_dat:
            if os.path.exists(file_dir) and remove == True:
                os.system('rm ' + file_dir)
            if os.path.exists(xml_dir) and remove == True:
                os.system('rm ' + xml_dir)
            if os.path.exists(kml_dir) and remove == True:
                os.system('rm ' + kml_dir)
            if os.path.exists(preview_dir) and remove == True:
                os.system('rm ' + preview_dir)

            invalid_files.append(file_dir)
        else:
            valid_files.append(file_dir)

    return invalid_files, valid_files


def sentinel_download(products=[], xml_only=False,  destination_folder='', project_folder='', user='', password=''):
    # Download the files which are found by the sentinel_available script.

    if not products:
        print 'No files to download'
        return

    wget_base = 'wget --retry-connrefused --waitretry=1 --read-timeout=20 --timeout=15 --continue --tries=20 --no-check-certificate --user=' + user + ' --password=' + password + ' '

    for product in products:
        date = str(product.findall('date')[1].text)
        date = datetime.datetime.strptime(date[:19], '%Y-%m-%dT%H:%M:%S')

        url = str('"'+product.findall('link')[0].attrib['href'][:-6]+ urllib.quote_plus('$value') +'"')
        name = str(product.find('title').text)

        track = str(product.find('int[@name="relativeorbitnumber"]').text)
        data_type = str(product.find(".//str[@name='filename']").text)[4:16]
        pol = str(product.find(".//str[@name='polarisationmode']").text).replace(' ', '')
        direction = str(product.find(".//str[@name='orbitdirection']").text)
        if direction == 'ASCENDING':
            direction = 'asc'
        elif direction == 'DESCENDING':
            direction = 'dsc'

        trackfolder = os.path.join(destination_folder, direction + '_t' + track.zfill(3))
        if not os.path.exists(trackfolder):
            os.mkdir(trackfolder)
        typefolder = os.path.join(trackfolder, data_type + '_' + pol)
        if not os.path.exists(typefolder):
            os.mkdir(typefolder)
        datefolder = os.path.join(typefolder, date.strftime('%Y%m%d'))
        if not os.path.exists(datefolder):
            os.mkdir(datefolder)

        xml_dir = os.path.join(datefolder, name + '.xml')
        file_dir = os.path.join(datefolder, name + '.SAFE.zip')
        kml_dir = os.path.join(datefolder, name + '.kml')
        preview_dir = os.path.join(datefolder, name + '.jpg')

        if project_folder:
            datefolder = os.path.join(project_folder, 's1', date.strftime('%Y%m%d') + '_t' + track)
            if not os.path.exists(datefolder):
                os.mkdir(datefolder)
            sentinel_folder = os.path.join(datefolder, 'sentinel_1')
            if not os.path.exists(sentinel_folder):
                os.mkdir(sentinel_folder)

            xml_project = os.path.join(datefolder, 'sentinel_1', name + '.xml')
            link_project = os.path.join(datefolder, 'sentinel_1', name + '.SAFE.zip')
            kml_project = os.path.join(datefolder, 'sentinel_1', name + '.kml')
            preview_project = os.path.join(datefolder, 'sentinel_1', name + '.jpg')

        # Save .xml files
        prod = etree.ElementTree(product)

        if not os.path.exists(xml_dir):
            prod.write(xml_dir, pretty_print = True)
        if project_folder:
            if not os.path.exists(xml_project):
                prod.write(xml_project, pretty_print = True)

        prev = "'preview'"
        png = "'quick-look.png'"
        kml = "'map-overlay.kml'"
        dat = "'" + name + ".SAFE'"

        preview_url = url[:-10] + '/Nodes(' + dat + ')/Nodes(' + prev + ')/Nodes(' + png + ')/' + urllib.quote_plus('$value') + '"'
        kml_url = url[:-10] + '/Nodes(' + dat + ')/Nodes(' + prev + ')/Nodes(' + kml + ')/' + urllib.quote_plus('$value') + '"'

        # Download data files and create symbolic link
        if xml_only == False: # So we also download the file
            if not os.path.exists(file_dir):
                wget_data = wget_base + url + ' -O ' + file_dir
                print('download url is:' + wget_data)
                os.system(wget_data)

                # Finally check whether the file is downloaded correctly. Otherwise delete file and wait for next round of
                # downloads.
                uuid = product.find('id').text
                valid = sentinel_quality_check(file_dir, uuid, user, password)
            else: # If the file already exists we assume it is valid.
                valid = True

            if valid == True:
                # First download additional files
                if not os.path.exists(preview_dir):
                    wget_preview = wget_base + preview_url + ' -O ' + preview_dir
                    os.system(wget_preview)
                if not os.path.exists(kml_dir):
                    wget_kml = wget_base + kml_url + ' -O ' + kml_dir
                    os.system(wget_kml)

                # Then copy to user folder and create links if project folder is used
                if project_folder:
                    if not os.path.exists(preview_project):
                        os.system('cp ' + preview_dir + ' ' + preview_project)
                    if not os.path.exists(kml_project):
                        os.system('cp ' + kml_dir + ' ' + kml_project)
                    if not os.path.exists(link_project):
                        os.system('ln -s ' + file_dir + ' ' + link_project)
            else:
                os.system('rm ' + file_dir)
                os.system('rm ' + xml_dir)
                os.system('rm ' + xml_project)

def sentinel_quality_check(filename, uuid, user, password):
    # Check whether the zip files can be unpacked or not. This is part of the download procedure.

    checksum_url = "https://scihub.copernicus.eu/dhus/odata/v1/Products('" + uuid + "')/Checksum/Value/" + urllib.quote_plus('$value')
    request = urllib2.Request(checksum_url)
    base64string = base64.b64encode('%s:%s' % (user, password))
    request.add_header("Authorization", "Basic %s" % base64string)

    # connect to server. Hopefully this works at once
    try:
        dat = urllib2.urlopen(request)
    except:
        print 'not possible to connect this time'
        return False

    html_dat = ''
    for line in dat:
        html_dat = html_dat + line

    # Check file on disk
    if sys.platform == 'darwin':
        md5 = subprocess.check_output('md5 ' + filename, shell=True)[-33:-1]
    elif sys.platform == 'linux2':
        md5 = subprocess.check_output('md5sum ' + filename, shell=True)[:32]
    else:
        'This function only works on mac or linux systems!'
        return False

    if md5 == html_dat.lower():
        return True
    else:
        return False

def download_orbits(start_date, end_date, pages=30, precise_folder='', restituted_folder =''):
    # This script downloads all orbits files from the precise orbits website, when pages is set to a very high number.
    # By default only the first page for the last two days (restituted) is checked.

    pages_res = min(pages, 60)
    pages_poe = pages # every day there are 8 restituted orbit files
    last_precise = '' # Last precise orbit file. Used to remove unused restituted orbit files.

    start_num = int(start_date[0:4] + start_date[5:7] + start_date[8:10])
    end_num = int(end_date[0:4] + end_date[5:7] + end_date[8:10])

    if precise_folder:
        for i in range(pages_poe):
            # First extract the orbitfiles from the page.

            url = 'https://qc.sentinel1.eo.esa.int/aux_poeorb/?page=' + str(i + 1)
            gcontext = ssl.SSLContext(ssl.PROTOCOL_TLSv1)
            try:
                page = urllib2.urlopen(url, context=gcontext)
            except TypeError:
                page = urllib2.urlopen(url)

            html = page.read().split('\n')
            orb_files = []

            for line in html:
                if re.search('<a .*href=.*>', line):
                    if re.search('EOF', line):
                        dat = re.search('<a href=.*>(.*)</a>', line)
                        orb_files.append(dat.group(1))

            if not last_precise:
                last_precise = orb_files[0]

            for orb in orb_files:
                # Download the orbit files
                filename = os.path.join(precise_folder, orb)

                if int(orb[42:50]) + 1 <= end_num and int(orb[42:50]) + 1 >= start_num:
                    url = 'https://qc.sentinel1.eo.esa.int/aux_poeorb/' + orb
                    if not os.path.exists(filename):
                        try:
                            urllib.urlretrieve(url, filename, context=gcontext)
                        except TypeError:
                            urllib.urlretrieve(url, filename)
                        print(orb + ' downloaded')
                    else:
                        print(orb + ' already downloaded')
                else:
                    print(orb + ' is out of date range')

            if len(orb_files) > 0:
                if int(orb[42:50]) < start_num:
                    break

    if restituted_folder:
        now = datetime.datetime.now()
        last_date = datetime.datetime.strptime(end_date, '%Y-%m-%d')
        diff = datetime.timedelta(days=25)

        print('Time difference to last date is ' + str((now - last_date).days))

        if now - last_date < diff:  # only run when precise orbits will not be available
            for i in range(pages_res):
                # First extract the orbitfiles from the page.

                url = 'https://qc.sentinel1.eo.esa.int/aux_resorb/?page=' + str(i + 1)
                gcontext = ssl.SSLContext(ssl.PROTOCOL_TLSv1)
                try:
                    page = urllib2.urlopen(url, context=gcontext)
                except TypeError:
                    page = urllib2.urlopen(url)

                html = page.read().split('\n')
                orb_files = []

                for line in html:
                    if re.search('<a .*href=.*>', line):
                        if re.search('EOF', line):
                            dat = re.search('<a href=.*>(.*)</a>', line)
                            orb_files.append(dat.group(1))

                for orb in orb_files:
                    # Download the orbit files
                    filename = os.path.join(precise_folder, orb)

                    if int(orb[42:50]) + 1 <= end_num and int(orb[42:50]) + 1 >= start_num:
                        url = 'https://qc.sentinel1.eo.esa.int/aux_poeorb/' + orb
                        if not os.path.exists(filename):
                            try:
                                urllib.urlretrieve(url, filename, context=gcontext)
                            except TypeError:
                                urllib.urlretrieve(url, filename)
                            print(orb + ' downloaded')
                        else:
                            print(orb + ' already downloaded')
                    else:
                        print(orb + ' is out of date range')

                if len(orb_files) > 0:
                    if int(orb[42:50]) < start_num:
                        break


# Actually execute the code...
if __name__ == "__main__":

    stack_folder = sys.argv[1]

    xml_file = os.path.join(os.path.join(stack_folder, 'doris_input.xml'))
    tree = ET.parse(xml_file)
    settings = tree.getroot()[0]
    print('reading xml file stack ' + xml_file)

    ROI = settings.find('.shape_file_path').text
    polarisation = settings.find('.polarisation').text
    archive_folder = settings.find('.sar_data_folder').text
    track = settings.find('.track').text
    orbit_folder = settings.find('.orbits_folder').text

    start_date = settings.find('.start_date').text
    end_date = settings.find('.end_date').text

    # Standard settings
    level = 'L1'
    sensor_mode = 'IW'
    product = 'SLC'

    # user settings
    xml_name = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    config_xml_file = os.path.join(os.path.join(xml_name, 'install', 'doris_config.xml'))
    print('reading xml file settings doris ' + config_xml_file)
    tree = ET.parse(config_xml_file)
    settings = tree.getroot()
    user = settings.find('.scihub_username').text
    password = settings.find('.scihub_password').text

    products, links, dates = sentinel_available(start_day=start_date, end_day=end_date, ROI=ROI,
                                                polarisation=polarisation, sensor_mode=sensor_mode, track=track,
                                                orbit_direction='', level=level, product=product,user=user,
                                                password=password)

    sentinel_download(products, destination_folder=archive_folder, user=user, password=password)

    precise_folder = os.path.join(orbit_folder, 'precise')
    if not os.path.exists(precise_folder):
        os.makedirs(precise_folder)
    restituted_folder = os.path.join(orbit_folder, 'restituted')
    if not os.path.exists(restituted_folder):
        os.makedirs(restituted_folder)

    download_orbits(start_date, end_date, pages=100, precise_folder=precise_folder, restituted_folder=restituted_folder)



# This function creates images based on complex or real input data.
# Input parameters are:
# - Input matrix
# - datatype
# - Image scaling (are pixels for example 1x2 km or 3x1 km?) scaling is azimuth / range.
# - Use of logscaling?
# - Plot amplitude / phase / both (not relevant for real values...)

# If you want to save your data as a geotiff or netcdf file use the read_write_data.py script. This will enable
# visualization in for example QGIS or ArcGIS
# If you want to do multilooking first apply the multilook.py script.

def create_image():



# This files defines a class for metadata objects of sentinel images. Large part of the work depends on python readers
# from the tops toolbox.

from doris.doris_stack.functions.xml_query import xml_query
import warnings
import os
from doris.doris_stack.main_code.burst import BurstMeta
from doris.doris_stack.functions.swath_metadata import burst_coverage, swath_coverage, swath_precise


class SwathMeta(object):
    # Class which stores and gathers information of a swath in a sentinel dataset

    def __init__(self, path='', swath_no='1', pol='vv', xml='', data=''):
        # Initialize object variables

        # This will contain a list of burst objects
        self.bursts = []

        # The following contain the path of xml and data file. Also swath_no and polarisation are given.
        self.swath_xml = ''
        self.swath_data = ''
        self.swath_no = ''
        self.swath_pol = ''

        # These variables contain the metadata and the convex hull of the bursts
        self.metadata = []
        self.coverage = []
        self.burst_centers = []
        self.burst_corners = []
        self.burst_shapes = []
        self.orbits = []
        self.orbit_type = ''

        # This function creates an swath object and searches for available data and xml files. It gives an error when
        # either the path does not exist, no data or xml files can be found or the data and xml files do not match.'
        if (not xml or not data) and not path:
            warnings.warn('Please provide either a product path or xml and data path')
            return

        if not xml or not data:
            warnings.warn('Data path is now used')

            xml_dir = os.path.join(path, 'annotation')
            xml = [f for f in os.listdir(xml_dir) if os.path.isfile(os.path.join(xml_dir, f))]

            data_dir = os.path.join(path, 'measurement')
            data = [f for f in os.listdir(data_dir) if os.path.isfile(os.path.join(data_dir, f))]

            # Select polarisation
            if not any(s in pol for s in ('hh','vv','hv','vh')):
                warnings.warn('Polarisation not recognized, using default (vv)')
                pol = 'vv'
            if not swath_no in ('1','2','3'):
                warnings.warn('Swath number not recognized, using default (1)')

            xml = [os.path.join(path,'annotation',x) for x in xml if x[12:14] in pol and x[6] == swath_no]
            data = [os.path.join(path,'measurement',x) for x in data if x[12:14] in pol and x[6] == swath_no]

        # Check if the data is there and if the filenames coincide.
        # print xml + str(len(xml))
        # print data + str(len(data))

        if type(xml) is str:
            xml = [xml]
        if type(data) is str:
            data = [data]
        if (len(xml) != 1 and type(xml) is list) or len(data) != 1:
            warnings.warn('Total number of files should be one!')
        if not os.path.exists(xml[0]) or not os.path.exists(data[0]):
            warnings.warn('Either xml or data path does not exist')
        if xml[0:-3] != data[0:-4]:
            warnings.warn('xml and data file do not correspond.')

        self.swath_xml = xml[0]
        self.swath_data = data[0]

    def meta_swath(self):
        # This function reads and stores metadata of different swaths in the swath objects.
        self.metadata = xml_query(self.swath_xml)
        corners, self.coverage = swath_coverage(self.metadata)
        self.burst_centers, self.burst_corners, self.burst_shapes = burst_coverage(self.metadata)


    def orbits_swath(self, precise_folder=''):
        # This functions loads the precise orbits for this swath
        if not precise_folder:
            print('xml information on orbit is used because no precise folder is specified')
            orbits, type_orb = swath_precise(self.metadata, precise_folder=precise_folder, dat_type='XML')
        else:
            orbits, type_orb = swath_precise(self.metadata, precise_folder=precise_folder, dat_type='POE')

        self.orbits = orbits
        self.orbit_type = type_orb

        return orbits, type_orb


    def meta_burst(self, precise_folder=''):
        # This function reads and stores metadata of different bursts in the bursts objects.

        if not self.metadata:
            self.meta_swath()
        if not self.orbits:
            self.orbits_swath(precise_folder=precise_folder)

        bursts_num = len(self.metadata['aux']['azimuthTimeStart'])

        if self.bursts:
            self.bursts = []

        for no in range(bursts_num):
            self.bursts.append(BurstMeta(path='',swath_no=self.swath_no, pol=self.swath_pol, burst_num=no + 1,
                                         xml=self.swath_xml, data=self.swath_data))
            self.bursts[no].burst_center = self.burst_centers[no][0]
            self.bursts[no].burst_coverage = self.burst_shapes[no]
            self.bursts[no].burst_corners = self.burst_corners[no]
            self.bursts[no].datapoints = self.orbits
            self.bursts[no].orbit_type = self.orbit_type



# Function created by Gert Mulder
# Institute TU Delft
# Date 9-11-2016
# Part of Doris 5.0

# This function will create the needed files for a datastack based on input information from the datastack.
import xml.etree.ElementTree as ET
import os
import pickle

# Test data
# settings_table = '/home/gert/software/doris/Doris_s1/sentinel_1/functions/inputfile_template.xml'
# inputfile_folder = '/media/gert/Data/dem/test/'
# sensor = 'sentinel-1'
# dem_info = inputfile_folder + 'output.dem.var'


class CreateInputFiles:
    # Input for the class to create inputfiles are:
    # - dem_info > variable or file with variable with information on the dem.
    # - settings_table > this table includes the general settings for different sensors. You can either choose one of
    #                       the predefined sensors or create a custom one.
    # - sensor > select the predefined sensor of your choice
    # - Other settings are mainly related to how much output information you want from the program. Also you can define
    #   the amount of memory used by the program.


    def __init__(self, dem_info ,settings_table, sensor):
        tree = ET.parse(settings_table)
        settings = tree.getroot()

        self.xml_data = settings.find('.' + sensor)
        self.header_data = self.xml_data.find('.header_settings')
        dem_info = open(dem_info, 'r')
        self.dem_var = pickle.load(dem_info)

        self.inputfilenames = ['coarsecorr', 'coarseorb', 'coherence', 'coherence_network', 'comprefdem', 'comprefpha', 'coregpm',
                          'dembased', 'finecoreg', 'geocode', 'interferogram', 'resample', 'subtrrefdem', 'subtrrefpha',
                          'unwrap', 'phasefilt']
        self.processes = ['coarsecorr', 'coarseorb', 'coherence', 'coherence', 'comprefdem', 'comprefpha', 'coregpm',
                          'dembased', 'finecoreg', 'geocode', 'interferogram', 'resample', 'subtrrefdem', 'subtrrefpha',
                          'unwrap', 'phasefilt']

    def create(self, inputfile_folder):
        for process_name, filename in zip(self.processes, self.inputfilenames):
            # Create file
            inputfilename = os.path.join(inputfile_folder, 'input.' + filename)
            txtfile = open(inputfilename, 'w')

            # Load xml data for processing step
            process = self.xml_data.find('./' + process_name + '/PROCESS')
            process_data = self.xml_data.find('.' + process_name)

            # Write to file
            txtfile = self._header(txtfile, self.header_data, process)
            txtfile = self._create_inputfiles(txtfile, process_data, self.dem_var)

            # Close file
            txtfile.close()

    def _create_inputfiles(self, txtfile, process_data, dem_var):
        # This functions calls the different inputfile creation scripts.

        for node in process_data:
            if not node.tag == 'PROCESS':
                if node.attrib['c'] == 'on':
                    c = 'c '
                else:
                    c = ''

                if 'var' in node.attrib and node.attrib['comment']:
                    txtfile.write(c + node.tag.ljust(20) + '\t' + dem_var[node.attrib['var']].ljust(20) + '\t // ' + node.attrib['comment'] + '\n')
                elif not 'var' in node.attrib and node.attrib['comment']:
                    txtfile.write(c + node.tag.ljust(20) + '\t' + node.text.ljust(20) + '\t // ' + node.attrib['comment'] + '\n')
                elif 'var' in node.attrib and not node.attrib['comment']:
                    txtfile.write(c + node.tag.ljust(20) + '\t' + dem_var[node.attrib['var']].ljust(20) + '\n')
                elif not 'var' in node.attrib and not node.attrib['comment']:
                    txtfile.write(c + node.tag.ljust(20) + '\t' + node.text.ljust(20) + '\n')

        txtfile.write("STOP                          \n")

        return txtfile

    def _header(self, txtfile, header_data, process):
        # Function to write header

        txtfile.write("c Inputfile created by Doris 5.0" + '\n')
        txtfile.write("c         " + "___".ljust(len(process.text) + 3) + "___ \n")
        txtfile.write("comment   ___" + process.text + "___ \n")
        txtfile.write("c                             \n")

        for node in header_data:
            if node.tag == 'PROCESS':
                txtfile.write("c \n")
                txtfile.write("PROCESS ".ljust(15) + process.text + " \n")
                txtfile.write("c \n")
            else:
                if node.attrib['comment']:
                    txtfile.write(node.tag.ljust(20) + '\t' + node.text.ljust(20) + '\t // ' + node.attrib['comment'] + '\n')

        txtfile.write("c                             \n")

        return txtfile



#!/usr/bin/env python

#from IPython.Debugger import Tracer; debug_here = Tracer()
#
#-----------------------------------------------------------------#
# A python code for parsing RS2 XML file into python data structures
# and from there into DORIS res file structure
#
# Author: TUDelft - 2011
# Maintainer: Piers Van der Torren
# License: GPL
#
#-----------------------------------------------------------------#

import os, re
from datetime import datetime, timedelta
#import argparse
import optparse

codeRevision=1.0   # this is the code revision number

def orbit_MDA(filename):
    """read MDA orbitfile and return generator with points
    """
    with open(filename) as orbfile:
        line = orbfile.readline().strip()
        while line != '':
            if len(line) == 24 and not line.startswith(';'):
                try:
                    pnt = [ datetime.strptime(line, '%Y-%j-%H:%M:%S.%f') ]
                    pnt.append(orbfile.readline().strip())
                    pnt.append(orbfile.readline().strip())
                    yield pnt
                except ValueError:
                    pass
            line = orbfile.readline().strip()

def orbit_filter(orbit,fromdate,todate):
    """filter orbitpoints and convert to DORIS orbit format
    """
    for pnt in orbit:
        if fromdate < pnt[0] < todate:
            yield('{0} {1} {2}'.format(pnt[0].hour*3600+pnt[0].minute*60+pnt[0].second+pnt[0].microsecond/1000000.0, pnt[1], pnt[2]))

def load_resfile(filename):
    """Load DORIS result file as nested dictionary with sections and key-value pairs
    """
    with open(filename) as resfile:
        resdata = resfile.read()

    # split res string in structures of the form:
    # .name: name of a brocessing block
    # .args: everything after _Start_<name>
    # .data: text between _Start_<name> and End_<name>
    # .status: everything after End_<name>
    regex = re.compile( r'\n[*]*[ \t]*\n[*]_Start_(?P<name>\w+):?[ \t]*' + \
                    r'(?P<args>[^\n]*)\n[*]*[ \t]*\n' + \
                    r'(?P<data>.*?)\n' + \
                    r'[*]*[ \t]*\n[*] End_\1:(?P<status>\w+)\n[*]*[ \t]*', re.DOTALL )
    
    # regex without star lines
    #regex = re.compile( r'\n[*]_Start_(?P<name>\w+):?[ \t]*' + \
    #                r'(?P<args>[^\n]*)\n' + \
    #                r'(?P<data>.*?)\n' + \
    #                r'[*] End_\1:(?P<status>\w+)\n', re.DOTALL )
        
    blocks = {}
    for block in regex.finditer( resdata ):
        blocks[block.group('name')] = block.group('data')
        for block2 in regex.finditer( block.group('data') ):
            blocks[block2.group('name')] = block.group('data')

    # split block.data in two fields:
    # #1=key, excluding the terminating colon
    # #2=value, leading whitespace stripped,
    # plus adjacent non key:value lines.
    regex = re.compile( r'\n(?P<key>[^\n:]+):' + \
                r'[ \t]*(?P<value>[^\n]*[^:]*)' + \
                r'(?=\n[^\n:]+:|\n[*])', re.DOTALL )

    res = {}
    for key in blocks:
        res[key] = {}
        for token in regex.finditer( blocks[key] ):
            pairkey = re.sub('[^a-zA-Z0-9]+', '', token.group('key'))
            res[key][pairkey] = token.group('value').strip()

    return res

def add_to_resfile(filename, section, data, args='', status='_NORMAL'):
    """Add new section to a DORIS result file.
    Write section header, data, and footer, and adjust process control flag.
    """
    with open(filename) as resfile:
        resdata = resfile.read()
    process_control = re.search(r'\n{0}:[ \t]*[01]'.format(section), resdata).span()[1]
    if resdata[process_control-1] != '0':
        raise Exception('already has precise orbits according to process control flags')

    header = '*'*67 + '\n*_Start_' + section + ':' + args + '\n' + '*'*67 + '\n'
    footer = '*'*67 + '\n* End_' + section + ':' + status + '\n' + '*'*67 + '\n'

    resdata = resdata[:process_control-1] + '1' + resdata[process_control:] + \
                        '\n' + header + data + '\n' + footer

    with open(filename, 'w') as resfile:
        resfile.write(resdata)

def clear_resfile_section(filename, section):
    """Clear a section in a DORIS result file.
    Remove the section, and set process control flag to 0 if it exists.
    """
    with open(filename) as resfile:
        resdata = resfile.read()

    process_control = re.search(r'\n{0}:[ \t]*[01]'.format(section), resdata)
    if process_control:
        process_control = process_control.span()[1]
        if resdata[process_control-1] == '1':
            resdata = resdata[:process_control-1] + '0' + resdata[process_control:]

    regex = re.compile( r'\n[*]*[ \t]*\n[*]_Start_{0}:?[ \t]*'.format(section) + \
                    r'(?P<args>[^\n]*)\n[*]*[ \t]*\n' + \
                    r'(?P<data>.*?)\n' + \
                    r'[*]*[ \t]*\n[*] End_{0}:(?P<status>\w+)\n[*]*[ \t]*\n'.format(section), re.DOTALL )

    sectionspan = regex.search(resdata).span()
    resdata = resdata[:sectionspan[0]] + resdata[sectionspan[1]:]

    with open(filename, 'w') as resfile:
        resfile.write(resdata)
    

def indate(str):
    """convert string to date, trying several formats
    """
    for format in ['%y%m%d%H%M%S', '%Y%m%d%H%M%S']:
        try:
            return datetime.strptime(str, format)
        except ValueError:
            pass
    raise ValueError('Date not understood: {0}'.format(str))

#def main():

# parse commandline arguments
parser = optparse.OptionParser(description='getorb-like tool extraction of radarsat orbits.')
parser.add_option('--dir',
                                     help='a directory containing RS2 orbits')
parser.add_option('--extratime', type=int, default=16,
                                     help='extra time before and after acquisition [s]')
parser.add_option('--resfile', 
                                     help='resfile to which to add precise orbits')
args = parser.parse_args()[0]

print('Getting precise orbits for Radarsat2\norbit dir: {dir}\nresfile: {resfile}\nextratime: {extratime}'\
    .format(dir=args.dir,extratime=args.extratime,resfile=args.resfile))
# read res file for neccesary metadata
res = load_resfile( args.resfile )

# get orbit number from line
# Scene identification:        Orbit: 08084    2009-07-02T05:52:34.496664Z 
orbit = int(res['readfiles']['Sceneidentification'][7:12])
# first pixel azimuth time
date = datetime.strptime(res['readfiles']['FirstpixelazimuthtimeUTC'], '%d-%b-%Y %H:%M:%S.%f')

orbitfile = os.path.join( args.dir, '{0:05d}_def.orb'.format(orbit) )
extratime = timedelta(seconds=args.extratime)
fromdate = date - extratime
todate = date + extratime + timedelta(seconds=4) # add 4 seconds as approximate acquisition time

# get orbit section
orbit_points = list(orbit_filter(orbit_MDA(orbitfile), fromdate, todate))

# save the orbit if more than 4 orbit points are found
if len(orbit_points) > 4:
    add_to_resfile( args.resfile, 'precise_orbits',
        't(s) X(m) Y(m) Z(m) X_V(m/s) Y_V(m/s) Z_V(m/s)\n' + \
        'NUMBER_OF_DATAPOINTS: {0}\n'.format(len(orbit_points)) + \
        '\n'.join(orbit_points) ) 

    # remove the leader orbit from the resfile
    clear_resfile_section( args.resfile, 'leader_datapoints')
else:
    print('No matching orbits found, tried orbit file {0}'.format(orbitfile))



#!/usr/bin/env python
import os,sys,time

def usage():
    print '\nUsage: python hhmmss2sec.py time'
    print '  where time in the form HH:MM:SS.sss .'
    print ' '
    print ' Example '
    print ' ./hhmmss2sec.py 16:36:40.393 '
    print '    59800.393000 '
    print ' '
    print ' See Also'
    print ' sec2hhmmss.py'
    
try:
    timeOfDay  = sys.argv[1] 
except:
    print 'Unrecognized input'
    usage()
    sys.exit(1)

timeOfDay= sys.argv[1].split(':')
hh  = float(timeOfDay[0]);
mm  = float(timeOfDay[1]);
ss  = float(timeOfDay[2]);

secOfDay=hh*3600+mm*60+ss
print "%f" %(secOfDay)



# The three functions here are used to transform coordinates and times in the original xml files to a usefull format for doris.
# Therefore these functions are generally used to read metadata of sentinel files.
import math
import numpy as np

def lph2xyz(line,pixel,container,norm_orbit_line,centroid_lon,centroid_lat,height):

    # initialization
    MAXITER=500
    CRITERPOS=1.0e-16
    # WGS84 Elliposid:
    ellipsoid=[6378137.0 , 6356752.3141]
    num_points = np.array([line]).shape[0]
    xyz       = np.zeros((num_points,3))
    #$$$ parameter of the image
    tr1  = float(container['rangeTimePix'][0])/2 # one way range time [sec]
    RSR  = float(container['rangeRSR'][0])*2 # one way in [HZ]
    centerphi = float(centroid_lat)
    centerlambda= float(centroid_lon)
    SOL=299792458
    # $$$ reference surace: WGS84

    ell_a  = ellipsoid[0]                  # semimajor of the ellipsoid
    ell_b  = ellipsoid[1]                  # semiminor of the ellipsoid
    ell_e2 = (ell_a**2-ell_b**2)/ell_a**2    # squared first eccentricity(derived)
    # $$$ ell_e2b=(ell_a^2-ell_b^2)/ell_b^2;  % squared second eccentricity(derived)

    # $$$ [lat,long,h] of scene center to [x,y,z]
    h            = height        # this is only for initial values

    centerphi    = centerphi*np.pi/180
    centerlambda = centerlambda*np.pi/180

    Ncenter      = ell_a/np.sqrt(1-ell_e2*(np.sin(centerphi)**2)) # eccentricity
    scenecenterx = (Ncenter+h)*np.cos(centerphi)*np.cos(centerlambda)
    scenecentery = (Ncenter+h)*np.cos(centerphi)*np.sin(centerlambda)
    scenecenterz = (Ncenter+h-ell_e2*Ncenter)*np.sin(centerphi)

    for n in range(0,num_points):   # loop through points

        posonellx = scenecenterx
        posonelly = scenecentery
        posonellz = scenecenterz
        ratime = tr1 + (pixel-1.0)/RSR

    #get position and velocity of the satellite
        possatx = norm_orbit_line[n,1]
        possaty = norm_orbit_line[n,2]
        possatz = norm_orbit_line[n,3]
        velsatx = norm_orbit_line[n,4]
        velsaty = norm_orbit_line[n,5]
        velsatz = norm_orbit_line[n,6]

        equationset=np.zeros((3,1))
        partialsxyz=np.zeros((3,3))
        for iter in range(1, MAXITER+1):

            #update equations and slove system
            dsat_Px = posonellx - possatx    #vector of 'satellite to P on ellipsoid'
            dsat_Py = posonelly - possaty
            dsat_Pz = posonellz - possatz

            equationset[0,0] = -(velsatx*dsat_Px+velsaty*dsat_Py+velsatz* dsat_Pz)

            equationset[1,0] = -(dsat_Px*dsat_Px+dsat_Py*dsat_Py+dsat_Pz*dsat_Pz-(SOL*ratime)**2)

            equationset[2,0] = -((posonellx*posonellx+posonelly*posonelly)/((ell_a+height)**2)+(posonellz/(ell_b+height))**2-1.0)

            partialsxyz[0,0] = velsatx
            partialsxyz[0,1] = velsaty
            partialsxyz[0,2] = velsatz
            partialsxyz[1,0] = 2*dsat_Px
            partialsxyz[1,1] = 2*dsat_Py
            partialsxyz[1,2] = 2*dsat_Pz
            partialsxyz[2,0] = (2*posonellx)/((ell_a+height)**2)
            partialsxyz[2,1] = (2*posonelly)/((ell_a+height)**2)
            partialsxyz[2,2] = (2*posonellz)/((ell_b+height)**2)

        # solve system [NOTE] orbit_normalized, otherwise close to
        # singular
            solpos = np.linalg.solve(partialsxyz,equationset)

            solx = solpos[0,0]
            soly = solpos[1,0]
            solz = solpos[2,0]

        # update solution
            posonellx = posonellx + solx
            posonelly = posonelly + soly
            posonellz = posonellz + solz

        # check convergence
            if (abs(solx)<CRITERPOS and abs(soly)<CRITERPOS and abs(solz)<CRITERPOS):
                break
            elif(iter>=MAXITER):
                MAXITER=MAXITER+1
        # final solution: array of XYZ coordinates
        xyz[n,:]=np.array([posonellx, posonelly, posonellz]).copy()

        return xyz


def xyz2ell(position):

    ellipsoid=[6378137.0 , 6356752.3141]
    ell_a = ellipsoid[0]
    ell_b = ellipsoid[1]

    ell_e2  = (ell_a**2-ell_b**2)/ell_a**2    #squared first eccentricity(derived)
    ell_e2b = (ell_a**2-ell_b**2)/ell_b**2    # squared second eccentricity(derived)

    posx = position[:,0]
    posy = position[:,1]
    posz = position[:,2]

    r    = math.sqrt(posx**2 + posy**2)
    mu   = math.atan2(posz*ell_a, r*ell_b)

    sin3 = (math.sin(mu))**3
    cos3 = (math.cos(mu))**3
    phi  = math.atan2((posz + ell_e2b * ell_b * sin3),(r - ell_e2 * ell_a* cos3))

    Radar_lambda = math.atan2(posy,posx)
    N = ell_a / math.sqrt(1 - ell_e2 * (math.sin(phi))**2) #for every point no
                                                # approx with scene.center
    height = (r/math.cos(phi)) - N

    phi_lam_height = np.zeros(3)
    phi_lam_height[0] = phi*180/math.pi
    phi_lam_height[1] = Radar_lambda*180/math.pi
    phi_lam_height[2] = height

    return phi_lam_height

def intrp_orbit(line,container,burst_number):

    intrpOrder = 'spline'
    orbit_time       = np.zeros(len(container['orbitTime']),dtype=np.float64)
    orbit_x = np.zeros(len(container['orbitTime']),dtype=np.float64)
    orbit_y = np.zeros(len(container['orbitTime']),dtype=np.float64)
    orbit_z = np.zeros(len(container['orbitTime']),dtype=np.float64)
    for row in range(len(container['orbitTime'])):

        #orbit_time_position=precorb[row]
        orbit_time[row]    =hms2sec(container['orbitTime'][row].split('T')[1])
        orbit_x[row]       =float( container['orbitX'][row])
        orbit_y[row]       =float( container['orbitY'][row])
        orbit_z[row]       =float( container['orbitZ'][row])

    # compute normalization factors
    px = orbit_time # time

    f  = min(px)
    g  = (max(px)-min(px))
    px = (px-f)/g
    # polyDegree
    polyDegree = 2

    coef_x1 = (np.polyfit(px,orbit_x,polyDegree));
    a = coef_x1[2]
    b = coef_x1[1]
    c = coef_x1[0]
    coef_x = [c/(g**2), b/g-(2*c*f)/(g**2), a-b*f/g+c*(f/g)**2]

    coef_y1 = (np.polyfit(px,orbit_y,polyDegree))
    a = coef_y1[2]
    b = coef_y1[1]
    c = coef_y1[0]
    coef_y = [c/(g**2), b/g-(2*c*f)/(g**2), a-b*f/g+c*(f/g)**2]

    coef_z1 = (np.polyfit(px,orbit_z,polyDegree));
    a = coef_z1[2]
    b = coef_z1[1]
    c = coef_z1[0]
    coef_z = [c/(g**2), b/g-(2*c*f)/(g**2), a-b*f/g+c*(f/g)**2]

    vel_x = np.polyval(np.polyder(coef_x),orbit_time)
    vel_y = np.polyval(np.polyder(coef_y),orbit_time)
    vel_z = np.polyval(np.polyder(coef_z),orbit_time)

    acc_x = np.kron(np.ones(len(container['orbitTime'])),np.polyder(np.polyder(coef_x)))
    acc_y = np.kron(np.ones(len(container['orbitTime'])),np.polyder(np.polyder(coef_y)))
    acc_z = np.kron(np.ones(len(container['orbitTime'])),np.polyder(np.polyder(coef_z)))
    #print 'acc_x.shape=',acc_x.shape

    # interpolated orbit
    norm_orbit = np.array([orbit_time, orbit_x,orbit_y,orbit_z,vel_x,  vel_y,  vel_z,acc_x,  acc_y,  acc_z]);

    # interpolated orbit for l_aztime
    PRF       = float(container['azimuthPRF'][0])   #[Hz]     # Pulse repeition frequency
    Taz_start = hms2sec(container['azimuthTimeStart'][burst_number].split('T')[1])

    ta1          = Taz_start  # start time in UTC [sec]

    l_aztime     = (line-1)/PRF + ta1
    pos_orbit_x = np.interp(l_aztime,orbit_time,orbit_x)
    pos_orbit_y = np.interp(l_aztime,orbit_time,orbit_y)
    pos_orbit_z = np.interp(l_aztime,orbit_time,orbit_z)

    vel_orbit_x = np.interp(l_aztime,orbit_time,vel_x)
    vel_orbit_y = np.interp(l_aztime,orbit_time,vel_y)
    vel_orbit_z = np.interp(l_aztime,orbit_time,vel_z)

    #acc = np.interp(orbit_time,[acc_x, acc_y, acc_z],    l_aztime,intrpOrder)
    acc_orbit_x = np.interp(l_aztime,orbit_time,acc_x)
    acc_orbit_y = np.interp(l_aztime,orbit_time,acc_y)
    acc_orbit_z = np.interp(l_aztime,orbit_time,acc_z)


    norm_orbit_line = np.array([l_aztime, pos_orbit_x,pos_orbit_y,pos_orbit_z,vel_orbit_x,vel_orbit_y,vel_orbit_z,acc_orbit_x,acc_orbit_y,acc_orbit_z])

    return norm_orbit.transpose(),norm_orbit_line.reshape(1,-1,order='F')

####################################################################################################

def hms2sec(hmsString,convertFlag='float'):
    # input hmsString syntax: XX:XX:XX.xxxxxx
    secString = int(hmsString[0:2])*3600 + \
        int(hmsString[3:5])*60 + \
        float(hmsString[6:])
    if convertFlag == 'int' :
        return int(secString)
    elif convertFlag == 'float' :
        return float(secString)
    else:
        return int(secString)


#!/usr/bin/env python

#-----------------------------------------------------------------#
# A python code for parsing CSK HDF5 file into python data structures
# and from there into DORIS res file structure
#
# Author: TUDelft - 2010
# Maintainer: Prabu Dheenathayalan
#
#-----------------------------------------------------------------#

import numpy, h5py, sys, math, time, string # numpy and h5py  required for HDF5 python support

codeRevision=1.0   # this code revision number

def usage():
    print 'INFO    : @(#)Doris InSAR software, $Revision: %s $, $Author: TUDelft $' % codeRevision
    print
    print 'Usage   : python csk_dump_header2doris.py csk_HDF5_product > OutputFileName'
    print '                               where csk_HDF5_product is the input filename'
    print
    print '          This software is part of Doris InSAR software package.\n'
    print '(c) 1999-2010 Delft University of Technology, the Netherlands.\n'

try:
  inputFileName  = sys.argv[1]
except:
    print '\nError   : Unrecognized input or missing arguments\n\n'
    usage()
    sys.exit(1)

# accessing the HDF5 product file
f = h5py.File(inputFileName, 'r')
s01 = f.get('/S01')
sbi = s01.get('SBI')
b001 = s01.get('B001')
qlk = s01.get('QLK')


if f.parent.__contains__('/') == False or f.parent.__contains__('/S01') == False or f.parent.__contains__('/S01/SBI') == False or f.parent.__contains__('/S01/B001') == False or f.parent.__contains__('/S01/QLK') == False :
   print 'ERROR: Wrong HDF5 format!'

# reading the attributes  
VolumeFile = f.attrs.__getitem__('Product Filename')
Volume_ID = f.attrs.__getitem__('Programmed Image ID') 
Volume_identifier = f.attrs.__getitem__('Product Specification Document') 
Volume_set_identifier='dummy'
NumberOfRecordsInRefFile='dummy'

SAR_PROCESSOR = f.attrs.__getitem__('L1A Software Version')
ProductTypeSpecifier = f.attrs.__getitem__('Product Type')
LogicalVolumeGeneratingFacility = f.attrs.__getitem__('Processing Centre')
LogicalVolumeCreationDate='dummy'
LocationAndDateTimeOfProductCreation = f.attrs.__getitem__('Product Generation UTC') 

SceneIdentification = f.attrs.__getitem__('Orbit Number')
orbitDir = f.attrs.__getitem__('Orbit Direction')
sceneMode = f.attrs.__getitem__('Acquisition Mode')
SceneLocation = 'dummy'
LeaderFile = f.attrs.__getitem__('Product Filename')
SAT_ID=f.attrs.__getitem__('Satellite ID')
SensorPlatformMissionIdentifier = f.attrs.__getitem__('Mission ID')
SceneCentreGeodeticCoordinates = f.attrs.__getitem__('Scene Centre Geodetic Coordinates')

Scene_centre_latitude=SceneCentreGeodeticCoordinates[0]
Scene_centre_longitude=SceneCentreGeodeticCoordinates[1]
Scene_centre_heading = f.attrs.__getitem__('Scene Orientation')
Radar_wavelength = f.attrs.__getitem__('Radar Wavelength')
Radar_frequency = f.attrs.__getitem__('Radar Frequency')
Swath_no = f.attrs.__getitem__('Subswaths Number')
Polarisation = s01.attrs.__getitem__('Polarisation')
ReferenceUTC = f.attrs.__getitem__('Reference UTC')
relZDAFTime = sbi.attrs.__getitem__('Zero Doppler Azimuth First Time') # precision of this variable need to be checked w.r.t tsx

if string.atoi(ReferenceUTC[20:])*10 != 0:
   ZDAFTime_msecsOfDay = round((string.atoi(ReferenceUTC[11:13])*60*60+string.atoi(ReferenceUTC[14:16])*60+string.atoi(ReferenceUTC[17:19])+pow(string.atoi(ReferenceUTC[20:])*10,(-(len(ReferenceUTC)-20)))+relZDAFTime)*1000)
else:
  ZDAFTime_msecsOfDay = round((string.atoi(ReferenceUTC[11:13])*60*60+string.atoi(ReferenceUTC[14:16])*60+string.atoi(ReferenceUTC[17:19])+relZDAFTime)*1000)

ZDAFTime_HH = math.floor(ZDAFTime_msecsOfDay/(60*60*1000))
ZDAFTime_MM = math.floor(ZDAFTime_msecsOfDay % (60*60*1000)/(60*1000))
ZDAFTime_SS = math.floor(ZDAFTime_msecsOfDay%(60*1000)/1000)
ZDAFTime_TTT = round(ZDAFTime_msecsOfDay%1000)

First_pixel_azimuth_time = str('%s %s:%s:%s.%s' % (time.strftime("%d-%b-%Y",time.strptime(ReferenceUTC.split()[0],"%Y-%m-%d")),str(int(ZDAFTime_HH)),str(int(ZDAFTime_MM)),str(int(ZDAFTime_SS)),str(int(ZDAFTime_TTT))))
PRF = s01.attrs.__getitem__('PRF')
Total_azimuth_band_width = s01.attrs.__getitem__('Azimuth Focusing Bandwidth')
Weighting_azimuth = f.attrs.__getitem__('Azimuth Focusing Weighting Function')

Xtrack_f_DC = f.attrs.__getitem__('Centroid vs Range Time Polynomial')
Xtrack_f_DC_constant = Xtrack_f_DC[0]
Xtrack_f_DC_linear = Xtrack_f_DC[1]
Xtrack_f_DC_quadratic = Xtrack_f_DC[2]
Range_time_to_first_pixel = sbi.attrs.__getitem__('Zero Doppler Range First Time')*1000

RSR = s01.attrs.__getitem__('Sampling Rate')/pow(10, 6)
Total_range_band_width = s01.attrs.__getitem__('Range Focusing Bandwidth')/pow(10, 6)
Weighting_range = f.attrs.__getitem__('Range Focusing Weighting Function')
NUMBER_OF_DATAPOINTS = f.attrs.__getitem__('Number of State Vectors')
StateVectorsTimes = f.attrs.__getitem__('State Vectors Times')

if string.atoi(ReferenceUTC[20:])*10 != 0:
   Time_Datapoints = (string.atoi(ReferenceUTC[11:13])*60*60+string.atoi(ReferenceUTC[14:16])*60+string.atoi(ReferenceUTC[17:19])+pow(string.atoi(ReferenceUTC[20:])*10,(-(len(ReferenceUTC)-20)))+StateVectorsTimes) % (60*60*24)
else:
  Time_Datapoints = (string.atoi(ReferenceUTC[11:13])*60*60+string.atoi(ReferenceUTC[14:16])*60+string.atoi(ReferenceUTC[17:19])+StateVectorsTimes) % (60*60*24)

ECEFSatellitePosition = f.attrs.__getitem__('ECEF Satellite Position')
ECEFSatelliteVelocity = f.attrs.__getitem__('ECEF Satellite Velocity')
nrows = len(ECEFSatellitePosition[:, 0])
ncols = len(ECEFSatellitePosition[0, :])
if nrows == 3:
   X_Datapoints=ECEFSatellitePosition[0,:]
   Y_Datapoints=ECEFSatellitePosition[1,:]
   Z_Datapoints=ECEFSatellitePosition[2,:]
   X_Velocity=ECEFSatelliteVelocity[0,:]
   Y_Velocity=ECEFSatelliteVelocity[1,:]
   Z_Velocity=ECEFSatelliteVelocity[2,:]
else:
   X_Datapoints=ECEFSatellitePosition[:,0]
   Y_Datapoints=ECEFSatellitePosition[:,1]
   Z_Datapoints=ECEFSatellitePosition[:,2]
   X_Velocity=ECEFSatelliteVelocity[:,0]
   Y_Velocity=ECEFSatelliteVelocity[:,1]
   Z_Velocity=ECEFSatelliteVelocity[:,2]

Datafile = f.attrs.__getitem__('Product Filename')


if sbi.shape[0]== 2:
   Number_of_lines_original = sbi.shape[2]
   Number_of_pixels_original = sbi.shape[1]
else:
   Number_of_lines_original = sbi.shape[0]
   Number_of_pixels_original = sbi.shape[1]


# ---------------------------------------------------------------------------------------------------------
# print the extracted attributes

dummyVar = 'DUMMY'

print('\ncsk_dump_header2doris.py v%s, doris software, 2010\n' % codeRevision)
print('*******************************************************************')
print('*_Start_readfiles:')
print('*******************************************************************')
print('Volume file:                                     %s' % VolumeFile)
print('Volume_ID:                                       %s' % Volume_ID)
print('Volume_identifier:                               %s' % Volume_identifier)
print('Volume_set_identifier:                           %s' % Volume_set_identifier)
print('(Check)Number of records in ref. file:           %s' % NumberOfRecordsInRefFile)
print('SAR_PROCESSOR:                                   %s L1A %s' % (SensorPlatformMissionIdentifier,SAR_PROCESSOR))
print('SWATH:                                           %s' % Swath_no)
print('PASS:                                            %s' % orbitDir)
print('IMAGING_MODE:                                    %s %s' % (sceneMode,Polarisation))
print('RADAR_FREQUENCY (Hz):                            %s' % Radar_frequency)
print('')
#print('Product type specifier:                          %s' % ProductTypeSpecifier) # returns SCS_B
print('Product type specifier:                          %s %s' % (SensorPlatformMissionIdentifier, ProductTypeSpecifier))
print('Logical volume generating facility:              %s' % LogicalVolumeGeneratingFacility)
print('Logical volume creation date:                    %s' % LogicalVolumeCreationDate)
print('Location and date/time of product creation:      %s' % LocationAndDateTimeOfProductCreation)

#print('Scene identification:                            Orbit: %s %s Mode: %s' % (SceneIdentification,orbitDir,sceneMode))
print('Scene identification:                            Orbit: %s' % SceneIdentification)
print('Scene location:                                  lat: %.4f lon: %.4f' % (float(Scene_centre_latitude),float(Scene_centre_longitude)))
print('')
print('Leader file:                                     %s' % LeaderFile)
#print('Sensor platform mission identifier:               %s' % SensorPlatformMissionIdentifier)
print('Sensor platform mission identifier:              %s' % SAT_ID)
print('Scene_centre_latitude:                           %s' % Scene_centre_latitude)
print('Scene_centre_longitude:                          %s' % Scene_centre_longitude)
print('Scene_centre_heading:                            %s' % Scene_centre_heading)
#print('Radar_wavelength (m):                            0.031') #HARDCODED!!!
print('Radar_wavelength (m):                            %s' % Radar_wavelength) 
print('First_pixel_azimuth_time (UTC):                  %s' % First_pixel_azimuth_time)
print('Pulse_Repetition_Frequency (computed, Hz):       %s' % PRF)
print('Total_azimuth_band_width (Hz):                   %s' % Total_azimuth_band_width)
print('Weighting_azimuth:                               %s' % string.upper(Weighting_azimuth))
print('Xtrack_f_DC_constant (Hz, early edge):           %s' % Xtrack_f_DC_constant)
print('Xtrack_f_DC_linear (Hz/s, early edge):           %s' % Xtrack_f_DC_linear)
print('Xtrack_f_DC_quadratic (Hz/s/s, early edge):      %s' % Xtrack_f_DC_quadratic)
print('Range_time_to_first_pixel (2way) (ms):           %0.15f' % (float(Range_time_to_first_pixel)))
print('Range_sampling_rate (computed, MHz):             %0.6f' % (float(RSR)))
print('Total_range_band_width (MHz):                    %s' % (float(Total_range_band_width)))
print('Weighting_range:                                 %s' % string.upper(Weighting_range))
print('')
print('*******************************************************************')
print('Datafile:                                        %s' % Datafile)
print('Dataformat:                                      %s' % 'HDF5')     # hardcoded!!!
print('Number_of_lines_original:                        %s' % str(Number_of_lines_original))
print('Number_of_pixels_original:                       %s' % str(Number_of_pixels_original))
print('*******************************************************************')
print('* End_readfiles:_NORMAL')
print('*******************************************************************')
print('')
print('')
print('*******************************************************************')
print('*_Start_leader_datapoints')
print('*******************************************************************')
print(' t(s)            X(m)            Y(m)            Z(m)            X_V(m/s)            Y_V(m/s)            Z_V(m/s)')
print('NUMBER_OF_DATAPOINTS:                    %s' % str(NUMBER_OF_DATAPOINTS))
print('')

for i in range(NUMBER_OF_DATAPOINTS):
    print(' %s %s %s %s %s %s %s' % (int(Time_Datapoints[i]),\
                                  X_Datapoints[i],\
                                  Y_Datapoints[i],\
                                  Z_Datapoints[i],\
				  X_Velocity[i],\
                                  Y_Velocity[i],\
                                  Z_Velocity[i]))

print('')
print('*******************************************************************')
print('* End_leader_datapoints:_NORMAL')
print('*******************************************************************')

#EOF



import os
import numpy as np
import warnings
from shutil import copyfile
from doris.doris_stack.main_code.resdata import ResData
import datetime
import subprocess


def baselines(dir_in,inputfile,start_date='2014-01-01',end_date='2018-01-01',doris=''):
    # This function calculates the baselines and plots a baseline plot.

    # Define doris path
    if not doris:
        doris = doris_path

    if not os.path.exists(dir_in):
        warnings.warn('The input directory does not exist!')
        return

    os.chdir(dir_in)
    process_folder = os.path.join(dir_in, 'baseline_process')
    if not os.path.exists(process_folder):
        os.mkdir(process_folder)
    os.chdir(process_folder)

    try:
        first = np.datetime64(start_date)
        last = np.datetime64(end_date)
    except:
        warnings.warn('Input dates could not be converted, use "yyyy-mm-dd"')
        return

    # Search for folders and take only the first burst.
    folders = next(os.walk(dir_in))[1]
    folders = sorted(folders)

    # Initialize... (Search for folders / resfiles / dates)
    n = 0
    res = []; date = []
    for fold in folders:
        # Select only the folders which has a name like yyyymmdd and are within
        if len(fold) == 8:
            # define date of folder
            date_prod = np.datetime64((fold[:4] + '-' + fold[4:6] + '-' + fold[6:]))

            if date_prod >= first and date_prod <= last:
                # Select the first swath
                date_fold = os.path.join(dir_in,fold)
                swath_fold = os.path.join(date_fold,next(os.walk(date_fold))[1][0])
                # Select the first burst
                prod_files = next(os.walk(swath_fold))[2]
                for file in prod_files:
                    if file.endswith('1.res'):
                        res.extend([os.path.join(swath_fold,file)])
                        date.extend([date_prod])
                        n = n + 1
                        break

    # Now create a set of baselines

    baselines = np.zeros([len(res),len(res)])
    resfiles = dict()

    # First create the ifgs.res files and store the data in a res data class.
    master = res[0]
    copyfile(master,os.path.join(process_folder,'master.res'))

    for resultfile, dat in zip(res, date):
        copyfile(resultfile,os.path.join(process_folder,'slave.res'))
        subprocess.call([doris + ' ' + inputfile], shell=True)

        dat = dat.astype(datetime.datetime).strftime('%Y-%m-%d')
        resfiles[dat] = ResData(type='interferogram',filename='ifgs.res')
        resfiles[dat].read()
        os.remove(os.path.join(process_folder,'ifgs.res'))

    # Then gather the baselines
    for dat, j in zip(date, range(len(date))):
        dat = dat.astype(datetime.datetime).strftime('%Y-%m-%d')
        baselines[j,0] = resfiles[dat].processes['coarse_orbits']['Bperp'][1]



    # Create figure of baselines.
    days = (date[0] - date).astype(float)
    plt.figure(111)
    plt.plot(baselines[:,0], days, marker='o')

    # Annotate
    for dat, x, y in zip(date, baselines[:,0], days):
        dat = dat.astype(datetime.datetime).strftime('%Y-%m-%d')
        plt.annotate(
            dat,
            xy = (x, y), xytext = (0, 0),
            textcoords = 'offset points', size = 8)

    plt.savefig('baseline_plot.pdf')




# Function created by Gert Mulder
# Institute TU Delft
# Date 9-11-2016
# Part of Doris 5.0

# This function creates a dem based on either a shape/kml file or a given bounding box. If a shape/kml file is given a
# minimum offset of about 0.1 degrees is used.
# All grids are based on the WGS84 projection. 
# Downloaded data is based on SRTM void filled data:
# Documentation: https://lpdaac.usgs.gov/sites/default/files/public/measures/docs/NASA_SRTM_V3.pdf

# Description srtm data: https://lpdaac.usgs.gov/dataset_discovery/measures/measures_products_table/SRTMGL1_v003
# Description srtm q data: https://lpdaac.usgs.gov/node/505

import numpy as np
import gdal
import gdalconst
import osr
from HTMLParser import HTMLParser
import pickle
import requests
import os
import zipfile
import fastkml
import shutil
from fiona import collection
import xml.etree.ElementTree as ET
import sys

# This are some example values...
# shape_filename = '/media/gert/Data/shapes/netherlands/netherland.shp'
# data_folder = '/media/gert/Data/dem/'
# dem_path = '/media/gert/Data/dem/test/'
# resample = 'regular_grid'
# doris_input = True
# quality = 'SRTM1'
# out_file = dem_path + 'output.dem'
# lats = np.arange(50.1, 53, 0.0005)
# lons = np.arange(2, 8, 0.0005)
# rounding = 1
# border = 0.1
# create_binary('', out_file, resample, doris_input, lats, lons, rounding, border, data_folder, quality)


class CreateDem:

    def __init__(self):
        return

    def create(self, shape_filename='', out_file='' ,var_file='', resample='regular_grid', doris_input=True, lats=[], lons=[],
                      rounding=1, border=0.1, data_folder='', quality='SRTM1', password='', username='', password_file=''):
        # This function stitches the different files together. If no data is available values will be zero. Which is
        # generally true because it is above sealevel.
        # The resampling is either
        # - none
        # - regular_grid (based on vectors of lats/lons)
        # - irregular_grid (based on lats/lons)

        if not out_file and not resample == 'irregular_grid':
            print('Please specify an output filename!')

        if shape_filename:
            latlim, lonlim = self._kml_shp_2_bb(shape_filename)
        else:
            try:
                latlim = [min(lats), max(lats)]
                lonlim = [min(lons), max(lons)]
            except:
                print('Not possible to create DEM grid.')
                return

        # Now add the rounding and borders to add the sides of our image
        # Please use rounding as a n/60 part of a degree (so 1/15 , 1/10 or 1/20 of a degree for example..)
        latlim = [np.floor((latlim[0] - border) / rounding) * rounding, np.ceil((latlim[1] + border) / rounding) * rounding]
        lonlim = [np.floor((lonlim[0] - border) / rounding) * rounding, np.ceil((lonlim[1] + border) / rounding) * rounding]

        # First download needed .hgt files. Quality is either 1, 3 or 30. If possible the files are downloaded. Otherwise
        # we fall back to lower quality. This is done using the elevation package
        tiles, q_tiles, tiles_30 = self._download_dem_files(latlim, lonlim, quality, data_folder, password=password, username=username)

        # Then create the final grid. This depends on the needed data type and possible resampling...
        if quality == 'SRTM1':
            pixel_degree = 3600
        elif quality == 'SRTM3':
            pixel_degree = 1200
        elif quality == 'SRTM30':
            pixel_degree = 120
        else:
            print('quality should be either SRTM1, SRTM3 or SRTM30!')
            return

        lat_size = int((latlim[1] - latlim[0]) * pixel_degree) + 1
        lon_size = int((lonlim[1] - lonlim[0]) * pixel_degree) + 1
        print('Bounding box is:')
        print('from ' + str(latlim[0]) + ' latitude to ' + str(latlim[1]))
        print('from ' + str(lonlim[0]) + ' longitude to ' + str(lonlim[1]))

        # Create final grid and add tiles.
        if quality == 'SRTM1' or quality == 'SRTM3':
            # Create file for DEM data
            bin_data = np.memmap(out_file, dtype=np.int16, shape=(lat_size, lon_size), mode='w+')
            bin_data = self._add_tiles(bin_data, tiles, quality, latlim, lonlim)

            # Create file for quality data
            bin_q_data = np.memmap(out_file + '.q', dtype=np.uint8, shape=(lat_size, lon_size), mode='w+')
            bin_q_data = self._add_tiles(bin_q_data, q_tiles, quality, latlim, lonlim, quality_file=True)

            # Save quality file to geotiff
            temp_q = os.path.join(data_folder, 'temp_q.tiff')
            n_latlim = [latlim[0] - 0.5 / pixel_degree, latlim[1] + 0.5 / pixel_degree]
            n_lonlim = [lonlim[0] - 0.5 / pixel_degree, lonlim[1] + 0.5 / pixel_degree]
            q_tiff = self._create_georeference(n_latlim, n_lonlim, 1.0 / pixel_degree, 1.0 / pixel_degree, 'uint8', temp_q)
            q_tiff.GetRasterBand(1).WriteArray(bin_q_data)
            q_tiff = None

        else:  # We are creating a 30 seconds map
            # Create file for DEM data
            bin_data = np.memmap(out_file, dtype=np.uint16, shape=(lat_size, lon_size))
            bin_data = self._add_tiles(bin_data, quality, tiles_30, latlim, lonlim)

            n_latlim = latlim
            n_lonlim = lonlim

        # Save the binary data as a geotiff
        print('Save data to geotiff')
        dem_tiff = os.path.join(data_folder, 'temp_dem.tiff')
        dem_data = self._create_georeference(n_latlim, n_lonlim, 1.0 / pixel_degree, 1.0 / pixel_degree, 'int16', dem_tiff)
        dem_data.GetRasterBand(1).WriteArray(bin_data)
        dem_data = None

        # First remove the egm96 from this raster.
        # Pre assign the egm tiff file first.
        print('Calculate geoid correction for SRTM data')
        egm_tiff = os.path.join(data_folder, 'egm96_resample.tiff')
        egm_data = self._create_georeference(n_latlim, n_lonlim, 1.0 / pixel_degree, 1.0 / pixel_degree, 'float32', egm_tiff)
        egm_data = None

        print('Correct DEM for geoid')
        self._add_egm96(dem_tiff, egm_tiff, data_folder)

        if resample == 'regular_grid':
            # If regular grid is used, we convert using gdal.
            # First create a geotiff file, then resample geotiff file. We always use cubic interpolation.
            print('Resampling to new regular grid')
            dlat = lats[1] - lats[0]
            dlon = lons[1] - lons[0]
            dem_tiff_final = os.path.join(data_folder, 'dem.tiff')
            dem_data_final = self._create_georeference(latlim, lonlim, dlat, dlon, 'float32', dem_tiff_final)
            dem_data_final = None
            dem_data_final = gdal.Open(dem_tiff_final, gdal.GA_Update)
            dem_data = gdal.Open(dem_tiff, gdal.GA_Update)

            gdal.ReprojectImage(dem_data, dem_data_final, None, None, gdal.GRA_Cubic)
            dem_data_final = None
            dem_tiff = dem_tiff_final

        elif resample == 'irregular_grid':
            # Use a simple bilinear approach to find values for specific points.
            print('Resampling to new irregular grid')
            heights = self._simple_bilinear(lats, lons, dem_tiff)

            return heights

        if doris_input == True:
            # Create a binary output file
            command = 'gdal_translate -of MFF ' + dem_tiff + ' ' + dem_tiff[:-5] + '.raw'
            os.system(command)

            if not os.path.exists(os.path.dirname(out_file)):
                os.makedirs(os.path.dirname(out_file))
            shutil.move(dem_tiff[:-5] + '.r00', out_file)

            # And create the scripts that go with them.
            self._output_doris_inputfiles(dem_tiff, out_file, var_file)

            # Finally also move .tiff file
            shutil.move(dem_tiff, out_file[:-4] + '.tiff')

        return


    def _add_tiles(self, outputdata, tiles, quality, latlim, lonlim, quality_file=False):
        # This function adds tiles to np.memmap file

        if quality == 'SRTM1':
            shape = (3601, 3601)
            s_size = 1.0 / 3600.0
            step_lat = 1
            step_lon = 1
        elif quality == 'SRTM3':
            shape = (1201, 1201)
            s_size = 1.0 / 1200.0
            step_lat = 1
            step_lon = 1
        elif quality == 'SRTM30':
            shape = (6000, 4800)
            s_size = 1.0 / 120.0
            step_lat = 50.0 - s_size
            step_lon = 40.0 - s_size
        else:
            print('quality should be either SRTM1, SRTM3 or SRTM30!')
            return

        print('total file size is ' + str(outputdata.shape[0]) + ' in latitude and ' + str(outputdata.shape[1]) + ' in longitude')

        for tile in tiles:
            if quality_file:
                image = np.fromfile(tile, dtype='>u1').reshape(shape)
            else:
                image = np.fromfile(tile, dtype='>i2').reshape(shape)

            if os.path.basename(tile)[7] == 'N':
                lat = float(os.path.basename(tile)[8:10])
            else:
                lat = - float(os.path.basename(tile)[8:10])
            if os.path.basename(tile)[10] == 'E':
                lon = float(os.path.basename(tile)[11:14])
            else:
                lon = - float(os.path.basename(tile)[11:14])
            if quality == 'SRTM30':
                lat = lat - 50 + (s_size / 2)
                lon += (s_size / 2)

            print('adding ' + tile)

            # Find the coordinates of the part of the tile that should be written to the output data.
            t_latlim = [max(lat, latlim[0]), min(lat + step_lat, latlim[1])]
            t_lonlim = [max(lon, lonlim[0]), min(lon + step_lon, lonlim[1])]
            t_latid = [shape[0] - int(round((t_latlim[0] - lat) / s_size)), shape[0] - (int(round((t_latlim[1] - lat) / s_size)) + 1)]
            t_lonid = [int(round((t_lonlim[0] - lon) / s_size)), int(round((t_lonlim[1] - lon) / s_size)) + 1]
            latsize = int(round((latlim[1] - latlim[0]) / s_size)) + 1
            latid = [latsize - int(round((t_latlim[0] - latlim[0]) / s_size)), latsize - (int(round((t_latlim[1] - latlim[0]) / s_size)) + 1)]
            lonid = [int(round((t_lonlim[0] - lonlim[0]) / s_size)), int(round((t_lonlim[1] - lonlim[0]) / s_size)) + 1]

            print('Adding tile lat ' + str(t_latid[1] + 1) + ' to ' + str(t_latid[0]) + ' into dem file ' +
                  str(latid[1] + 1) + ' to ' + str(latid[0]))
            print('Adding tile lon ' + str(t_lonid[0] + 1) + ' to ' + str(t_lonid[1]) + ' into dem file ' +
                  str(lonid[0] + 1) + ' to ' + str(lonid[1]))

            # Assign values from tiles to outputdata
            if quality == 'SRTM30':
                outputdata[latid[1]: latid[0]-2, lonid[0]: lonid[1]-2] = image[t_latid[1]: t_latid[0]-2, t_lonid[0]: t_lonid[1]-2]
            else:
                outputdata[latid[1]: latid[0]-1, lonid[0]: lonid[1]-1] = image[t_latid[1]: t_latid[0]-1, t_lonid[0]: t_lonid[1]-1]

        return outputdata


    def _download_dem_files(self, latlim, lonlim, quality, data_folder, username='', password='', password_xml=''):
        # This function downloads data either in 1,3 or 30 arc seconds. When you choose either 1 or 3 seconds also 30
        # seconds is downloaded to fill the voids.
        # In this operation the needed files are also extracted...

        # Check the username and password and load from config file if needed.
        if not username or not password:
            if os.path.exists(password_xml):
                tree = ET.parse(password_xml)
                settings = tree.getroot()

                username = settings.find('.usgs_username').text
                password = settings.find('.usgs_password').text
            else:
                print('You should specify a username and password to download SRTM data. ')
                return

        filelist = self._srtm_listing(data_folder, username=username, password=password)
        outfiles = []
        q_files = []
        outfiles_30 = []

        lats = np.arange(np.floor(latlim[0]), np.ceil(latlim[1]))
        lons = np.arange(np.floor(lonlim[0]), np.ceil(lonlim[1]))

        if quality == 'SRTM1' or quality == 'SRTM3':
            for lat in lats:
                for lon in lons:

                    lat = int(lat)
                    lon = int(lon)

                    if lat < 0:
                        latstr = 'S' + str(abs(lat)).zfill(2)
                    else:
                        latstr = 'N' + str(lat).zfill(2)
                    if lon < 0:
                        lonstr = 'W' + str(abs(lon)).zfill(3)
                    else:
                        lonstr = 'E' + str(lon).zfill(3)

                    # Check if file exists in filelist
                    if str(lat) not in filelist[quality]:
                        continue
                    elif str(lon) not in filelist[quality][str(lat)]:
                        continue

                    filename = os.path.join(data_folder, latstr + lonstr + 'SRTMGL3.hgt.zip')
                    q_file = os.path.join(data_folder, latstr + lonstr + 'SRTMGL3.q.zip')
                    extracted_file = os.path.join(data_folder, quality + '__' + latstr + lonstr + '.hgt')
                    q_extracted = os.path.join(data_folder, quality + '__' + latstr + lonstr + '.q')

                    if not os.path.exists(extracted_file) or not os.path.exists(q_extracted):
                        # Download and unzip
                        download_dem = filelist[quality][str(lat)][str(lon)]
                        download_q = download_dem[:-7] + 'num.zip'

                        command = 'wget ' + download_dem + ' --user ' + username + ' --password ' + password + ' -O ' + filename
                        q_command = 'wget ' + download_q + ' --user ' + username + ' --password ' + password + ' -O ' + q_file
                        try:
                            os.system(command)
                            zip_data = zipfile.ZipFile(filename)
                            source = zip_data.open(zip_data.namelist()[0])
                            target = open(extracted_file, 'wb')
                            shutil.copyfileobj(source, target, length=-1)
                            target.close()
                            outfiles.append(extracted_file)
                            os.remove(filename)

                            os.system(q_command)
                            zip_data = zipfile.ZipFile(q_file)
                            source = zip_data.open(zip_data.namelist()[0])
                            target = open(q_extracted, 'wb')
                            shutil.copyfileobj(source, target, length=-1)
                            target.close()
                            q_files.append(q_extracted)
                            os.remove(q_file)
                        except:
                            print('Failed to download or process ' + filename)

                    else:
                        outfiles.append(extracted_file)
                        q_files.append(q_extracted)

        for lat in lats:
            for lon in lons:
                # Now check with which 30 seconds file this corresponds
                lat50 = int(np.floor(float(lat + 10) / 50)) * 50 + 40
                lon40 = int(np.floor(float(lon + 20) / 40)) * 40 - 20

                if lat50 < 0:
                    latstr = 'S' + str(abs(lat50)).zfill(2)
                else:
                    latstr = 'N' + str(lat50).zfill(2)
                if lon40 < 0:
                    lonstr = 'W' + str(abs(lon40)).zfill(3)
                else:
                    lonstr = 'E' + str(lon40).zfill(3)

                # Does this file exist..
                if str(lat50) not in filelist['SRTM30']:
                    continue
                elif str(lon40) not in filelist['SRTM30'][str(lat50)]:
                    continue

                filename = os.path.join(data_folder, latstr + lonstr + 'SRTMGL3.hgt.zip')
                extracted_file = os.path.join(data_folder, 'SRTM30_' + latstr + lonstr + '.hgt')

                # Download and unzip file if possible and add to list
                if not os.path.exists(extracted_file):
                    # Download and unzip
                    user = 'USERNAME'
                    password = 'PASSWORD'

                    command = 'wget ' + filelist['SRTM30'][str(lat50)][str(lon40)] + ' --user ' + user + ' --password ' + password + \
                              ' -O ' + filename
                    try:
                        os.system(command)
                        zip_data = zipfile.ZipFile(filename)
                        source = zip_data.open(zip_data.namelist()[0])
                        target = open(extracted_file, 'wb')
                        shutil.copyfileobj(source, target, length=-1)
                        target.close()
                        outfiles_30.append(extracted_file)

                    except:
                        print('Failed to download or process ' + filename)
                elif extracted_file not in outfiles_30:
                    outfiles_30.append(extracted_file)

        return outfiles, q_files, outfiles_30


    def _simple_bilinear(self, lats, lons, dem_tiff, data_folder):
        # To resample to datapoints, we convert the tiff file to a binary file
        bin_file = os.path.join(data_folder, 'dem.raw')
        command = 'gdal_translate -of MFF ' + dem_tiff + ' ' + bin_file
        os.system(command)
        shutil.move(bin_file[:-4] + '.r00', bin_file)

        # Read it as a memmap function
        dem_data = gdal.Open(dem_tiff, gdal.GA_Update)
        size = (dem_data.RasterYSize, dem_data.RasterXSize)
        data = np.memmap(bin_file, shape=size, dtype=np.dtype('float32'))
        r = dem_data.GetGeoTransform()

        # And find the values of corresponding points using bilinear interpolation.
        x_id = np.floor((lons - r[0]) / r[1]).astype('int32')
        x_diff = (((lons - r[0]) / r[1]) - x_id)
        y_id = np.floor((lats - r[3]) / r[5]).astype('int32')
        y_diff = (((lats - r[3]) / r[5]) - y_id)

        # Calculate final results
        ll_cont = data[y_id, x_id] * (1-x_diff) * (1-y_diff)
        ul_cont = data[y_id + 1, x_id] * (1-x_diff) * y_diff
        ur_cont = data[y_id + 1, x_id + 1] * x_diff * y_diff
        lr_cont = data[y_id, x_id + 1] * x_diff * (1-y_diff)
        heights = ll_cont + ul_cont + ur_cont + lr_cont

        return heights


    def _kml_shp_2_bb(self, filename):
        # from kml and shape file to a bounding box. We will always use a bounding box to create the final product.

        if filename.endswith('.shp'):
            with collection(filename, "r") as inputshape:

                shapes = [shape for shape in inputshape]
                # only first shape
                dat = shapes[0]['geometry']['coordinates']

                lon = [l[0] for l in dat[0]]
                lat = [l[1] for l in dat[0]]

                latlim = [min(lat), max(lat)]
                lonlim = [min(lon), max(lon)]

        elif filename.endswith('.kml'):
            doc = file(filename).read()
            k = fastkml.KML()
            k.from_string(doc)
            dat = list(list(k.features())[0].features())[0].geometry[0].exterior.coords[:]

            lon = [l[0] for l in dat[0]]
            lat = [l[1] for l in dat[0]]

            latlim = [min(lat), max(lat)]
            lonlim = [min(lon), max(lon)]
        else:
            print 'format not recognized! Pleas creat either a .kml or .shp file.'
            return []

        return latlim, lonlim


    def _add_egm96(self, dem_tiff, egm_tiff, data_folder):
        # This function adds the geoid from egm96

        # Load egm96 grid and resample to input grid using gdal.
        # (For this purpose the grid is downloaded from:
        # http://earth-info.nga.mil/GandG/wgs84/gravitymod/egm96/binary/binarygeoid.html
        # In principle this is converted to geotiff here,

        filename = os.path.join(data_folder, 'EGM96_15min.dat')
        egm_source_tiff = os.path.join(data_folder, 'EGM96_15min.tiff')

        if not os.path.exists(egm_source_tiff):
            if not os.path.exists(filename):
                # Download egm96 file
                command = 'wget http://earth-info.nga.mil/GandG/wgs84/gravitymod/egm96/binary/WW15MGH.DAC -O ' + filename
                os.system(command)

            # Get georeference
            latlim = [-90.125, 90.125]
            lonlim = [-0.125, 359.875]
            dlat = 0.25
            dlon = 0.25
            egm_data = self._create_georeference(latlim, lonlim, dlat, dlon, 'float32', egm_source_tiff)

            # Load data
            egm96 = np.fromfile(filename, dtype='>i2').reshape((721, 1440)).astype('float32')

            # Save as geotiff
            egm_data.GetRasterBand(1).WriteArray(egm96 / 100)
            egm_data = None

        egm_source = gdal.Open(egm_source_tiff, gdal.GA_Update)
        egm_data = gdal.Open(egm_tiff, gdal.GA_Update)

        # Now resample to new dataset. (Copy information from gdal_dem)
        gdal.ReprojectImage(egm_source, egm_data, None, None, gdalconst.GRA_Bilinear)
        egm_data = None

        dem_new = dem_tiff + '.new'

        # Finally open original dataset and subtract
        command = 'gdal_calc.py -A ' + dem_tiff + ' -B ' + egm_tiff + ' --outfile=' + dem_new + ' --calc="A+B"'
        os.system(command)

        shutil.move(dem_new, dem_tiff)


    def _output_doris_inputfiles(self, dem_tiff, out_file, var_file):
        # This script outputs doris inputfiles which can be copied for further processing.

        dem = gdal.Open(dem_tiff)

        xsize = dem.RasterXSize
        ysize = dem.RasterYSize
        georef = dem.GetGeoTransform()
        dlat = georef[1]
        dlon = abs(georef[5])
        lonmin = georef[0] + (dlon * 0.5)
        latmax = georef[3] - (dlat * 0.5)

        output_txt = out_file + '.doris_inputfile'
        output_var = var_file
        output_var = open(output_var, 'w')
        txtfile = open(output_txt, 'w')

        dem_var = dict()
        dem_var['in_dem'] = out_file
        dem_var['in_format'] = 'r4'
        dem_var['in_size'] = str(ysize) + " " + str(xsize)
        dem_var['in_delta'] = str(dlat) + " " + str(dlon)
        dem_var['in_ul'] = str(latmax) + " " + str(lonmin)
        dem_var['in_nodata'] = '-32768'
        pickle.dump(dem_var, output_var)
        output_var.close()

        txtfile.write("# The processing cards generated by $(basename $0) script. \n")
        txtfile.write("# Using parameters: $@ \n")
        txtfile.write('# Copy the section(s) that is/are necessary to your processing setup. \n')
        txtfile.write("c         ___             ___ \n")
        txtfile.write("comment   ___SIM AMPLITUDE___ \n")
        txtfile.write("c                             \n")
        txtfile.write("SAM_IN_DEM     " + out_file + " \n")
        txtfile.write("SAM_IN_FORMAT   r4 \t\t\t // default is short integer \n")
        txtfile.write("SAM_IN_SIZE    " + str(ysize) + " " + str(xsize) + " \n")
        txtfile.write("SAM_IN_DELTA   " + str(dlat) + " " + str(dlon) + " \n")
        txtfile.write("SAM_IN_UL      " + str(latmax) + " " + str(lonmin) + " \n")
        txtfile.write("SAM_IN_NODATA  -32768 \n")
        txtfile.write("  \n")
        txtfile.write("  \n")
        txtfile.write("c         ___          ___ \n")
        txtfile.write("comment   ___DEM ASSIST___ \n")
        txtfile.write("c                             \n")
        txtfile.write("DAC_IN_DEM     $dempath/$outfile5 \n")
        txtfile.write("DAC_IN_FORMAT   r4 \t\t\t // default is short integer \n")
        txtfile.write("DAC_IN_SIZE    " + str(ysize) + " " + str(xsize) + " \n")
        txtfile.write("DAC_IN_DELTA   " + str(dlat) + " " + str(dlon) + " \n")
        txtfile.write("DAC_IN_UL      " + str(latmax) + " " + str(lonmin) + " \n")
        txtfile.write("DAC_IN_NODATA  -32768 \n")
        txtfile.write("  \n")
        txtfile.write("  \n")
        txtfile.write("c         ___             ___ \n")
        txtfile.write("comment   ___REFERENCE DEM___ \n")
        txtfile.write("c                             \n")
        txtfile.write("## CRD_METHOD   DEMINTRPMETHOD \n")
        txtfile.write("CRD_IN_DEM     $dempath/$outfile5 \n")
        txtfile.write("CRD_IN_FORMAT   r4 \t\t\t // default is short integer \n")
        txtfile.write("CRD_IN_SIZE    " + str(ysize) + " " + str(xsize) + " \n")
        txtfile.write("CRD_IN_DELTA   " + str(dlat) + " " + str(dlon) + " \n")
        txtfile.write("CRD_IN_UL      " + str(latmax) + " " + str(lonmin) + " \n")
        txtfile.write("CRD_IN_NODATA  -32768 \n")

        txtfile.close()


    def _srtm_listing(self, data_folder, username, password):
        # This script makes a list of all the available 1,3 and 30 arc second datafiles.
        # This makes it easier to detect whether files do or don't exist.

        data_file = os.path.join(data_folder, 'filelist')
        if os.path.exists(data_file):
            dat = open(data_file, 'r')
            filelist = pickle.load(dat)
            dat.close()
            return filelist

        server = "http://e4ftl01.cr.usgs.gov"

        folders = 'SRTM/SRTMGL1.003/2000.02.11/', 'SRTM/SRTMGL3.003/2000.02.11/', 'SRTM/SRTMGL30.002/2000.02.11/'
        keys = ['SRTM1', 'SRTM3', 'SRTM30']
        filelist = dict()
        filelist['SRTM1'] = dict()
        filelist['SRTM3'] = dict()
        filelist['SRTM30'] = dict()

        for folder, key_value in zip(folders, keys):

            conn = requests.get(server + '/' + folder, auth=(username, password))
            if conn.status_code == 200:
                print "status200 received ok"
            else:
                print "an error occurred during connection"

            data = conn.text
            parser = parseHTMLDirectoryListing()
            parser.feed(data)
            files = parser.getDirListing()

            if key_value == 'SRTM1' or key_value == 'SRTM3':
                files = [f for f in files if f.endswith('hgt.zip')]
                north = [int(filename[1:3]) for filename in files]
                east = [int(filename[4:7]) for filename in files]
                for i in [i for i, filename in enumerate(files) if filename[0] == 'S']:
                    north[i] = north[i] * -1
                for i in [i for i, filename in enumerate(files) if filename[3] == 'W']:
                    east[i] = east[i] * -1
            else:
                files = [f for f in files if f.endswith('dem.zip')]
                north = [int(filename[5:7]) for filename in files]
                east = [int(filename[1:4]) for filename in files]
                for i in [i for i, filename in enumerate(files) if filename[4] == 's']:
                    north[i] = north[i] * -1
                for i in [i for i, filename in enumerate(files) if filename[0] == 'w']:
                    east[i] = east[i] * -1

            for filename, n, e in zip(files, north, east):
                if not str(n) in filelist[key_value]:
                    filelist[key_value][str(n)] = dict()
                filelist[key_value][str(n)][str(e)] = server + '/' + folder + filename

        file_list = open(os.path.join(data_folder, 'filelist'), 'w')
        pickle.dump(filelist, file_list)
        file_list.close()

        return filelist


    def _create_georeference(self, latlim, lonlim, dlat, dlon, dtype='int16', filename=''):
        # This function also creates a dem file but creates an geotiff file instead. This functionality is used to allow
        # resampling in later steps using gdal.

        conversion = {
            "uint8": 1,
            "int8": 1,
            "uint16": 2,
            "int16": 3,
            "uint32": 4,
            "int32": 5,
            "float32": 6,
            "float64": 7,
            "complex64": 10,
            "complex128": 11,
        }

        if filename:
            driver = gdal.GetDriverByName('Gtiff')
            dataset = driver.Create(filename,
                                    int(np.round((lonlim[1] - lonlim[0]) / dlon)),
                                    int(np.round((latlim[1] - latlim[0]) / dlat)),
                                    1,
                                    conversion[dtype], ['COMPRESS=LZW', 'BIGTIFF=YES'])
        else:
            driver = gdal.GetDriverByName('mem')
            dataset = driver.Create('',
                                    int((lonlim[1] - lonlim[0]) / dlon),
                                    int((latlim[1] - latlim[0]) / dlat),
                                    1,
                                    conversion[dtype])

        dataset.SetGeoTransform((
            lonlim[0],  # 0
            dlat,  # 1
            0,  # 2
            latlim[1],  # 3
            0,  # 4
            -dlon))

        spatial_ref = osr.SpatialReference()
        spatial_ref.ImportFromEPSG(4326)

        dataset.SetProjection(spatial_ref.ExportToWkt())

        return dataset

    # This function is not used because data does not contain gaps....
    def _fill_voids(self, outputdata, output_30sec, quality):
        # This function fills the voids in our output data.

        if quality == 'SRTM1':
            s_size = 1.0 / 3600.0
        elif quality == 'SRTM3':
            s_size = 1.0 / 1200.0
        else:
            print('quality should be either SRTM1 or SRTM3!')
            return

        # This script assumes that the final dem is at least divided in 1 arc minute blocks so the two input fields are
        # perfectly alligned.
        id_void = np.argwhere(outputdata == -32767)

        if id:  # If there are any voids...
            id_30 = np.floor(id_void * 120.0 * s_size)
            outputdata[id_void] = output_30sec[id_30]

        return outputdata

# Following code is adapted from srtm-1.py > downloaded from
# https://svn.openstreetmap.org/applications/utils/import/srtm2wayinfo/python/srtm.py
class parseHTMLDirectoryListing(HTMLParser):
    def __init__(self):
        # print "parseHTMLDirectoryListing.__init__"
        HTMLParser.__init__(self)
        self.title = "Undefined"
        self.isDirListing = False
        self.dirList = []
        self.inTitle = False
        self.inHyperLink = False
        self.currAttrs = ""
        self.currHref = ""

    def handle_starttag(self, tag, attrs):
        # print "Encountered the beginning of a %s tag" % tag
        if tag == "title":
            self.inTitle = True
        if tag == "a":
            self.inHyperLink = True
            self.currAttrs = attrs
            for attr in attrs:
                if attr[0] == 'href':
                    self.currHref = attr[1]

    def handle_endtag(self, tag):
        # print "Encountered the end of a %s tag" % tag
        if tag == "title":
            self.inTitle = False
        if tag == "a":
            # This is to avoid us adding the parent directory to the list.
            if self.currHref != "":
                self.dirList.append(self.currHref)
            self.currAttrs = ""
            self.currHref = ""
            self.inHyperLink = False

    def handle_data(self, data):
        if self.inTitle:
            self.title = data
            print "title=%s" % data
            if "Index of" in self.title:
                # print "it is an index!!!!"
                self.isDirListing = True
        if self.inHyperLink:
            # We do not include parent directory in listing.
            if "Parent Directory" in data:
                self.currHref = ""

    def getDirListing(self):
        return self.dirList

# Actually execute the code...
if __name__ == "__main__":

    stack_folder = sys.argv[1]
    if len(sys.argv) > 2:
        quality = sys.argv[2]
        if quality not in ['SRTM1', 'SRTM3']:
            quality = 'SRTM3'
    else:
        quality = 'SRTM3'

    xml_file = os.path.join(os.path.join(stack_folder, 'doris_input.xml'))
    print('reading xml file stack ' + xml_file)
    tree_stack = ET.parse(xml_file)
    settings_stack = tree_stack.getroot()[0]

    xml_file = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'install',
                                 'doris_config.xml')
    print('reading xml file settings doris ' + xml_file)
    tree_doris = ET.parse(xml_file)
    settings_doris = tree_doris.getroot()

    # Then create the dem file
    shapefile = settings_stack.find('.shape_file_path').text
    dem_calc_folder = settings_stack.find('.dem_processing_folder').text
    dem_out_folder = settings_stack.find('.dem_folder').text

    dem_out = os.path.join(dem_out_folder, 'dem.raw')
    dem_var = os.path.join(dem_out_folder, 'var.raw')

    srtm_username = settings_doris.find('.usgs_username').text
    srtm_password = settings_doris.find('.usgs_password').text

    dem = CreateDem()
    dem.create(shapefile, dem_out, dem_var, resample=None,
            doris_input=True, rounding=1, border=1,
            data_folder=dem_calc_folder, quality=quality,
            password=srtm_password, username=srtm_username)



from setuptools import setup

setup(
    name='doris',
    version='5.0.3',
    packages=['install', 'doris_stack', 'doris_stack.functions', 'doris_stack.main_code', 'prepare_stack'],
    url='https://github.com/TUDelftGeodesy/Doris',
    license='LICENSE.txt',
    author='Gert Mulder',
    author_email='g.mulder-@tudelft.nl',
    description='doris InSAR processing software',
    install_requires=['numpy', 'shapely', 'requests', 'fiona', 'gdal', 'osr', 'scipy', 'fastkml']
)



# This file is created by:
# Gert Mulder
# TU Delft
# 04-07-2016
##############################

import sys
from doris.doris_stack.main_code.stack import StackData
from doris.doris_stack.main_code.dorisparameters import DorisParameters
from doris.doris_stack.main_code.grs_profile import GRS_Profile
from doris.doris_stack.main_code.single_master_stack import SingleMaster

class DorisSentinel1(object):

    def run(self, doris_parameters_path, start_date, end_date, master_date):

        print 'start sentinel 1 processing'

        #Set your input variables here. You should use absolute paths.
        dorisParameters = DorisParameters(doris_parameters_path)
        # fix DLE
        sys.path.extend([dorisParameters.function_path])

        profile = GRS_Profile(dorisParameters.profile_log + '_' + str(dorisParameters.nr_of_jobs), dorisParameters.verbose)

        # The shapefile to select the area of interest. You can easily find a shapefile countries or regions on the internet.
        # For example via diva-gis.org. Shapefile for the Netherlands can be found in the same folder under shapes.
        shape_dat = dorisParameters.shape_dat # '/...../test.shp'

        # The folder where SLC images from a specific track are stored. This data will be used as input for the script
        track_dir = dorisParameters.track_dir # '/......'

        # This is the output folder.
        stack_path = dorisParameters.stack_path  #'/.....'

        # Folder where the precise or restituted orbits are stored. Precise orbits can be found via the following link:
        # 'https://qc.sentinel1.eo.esa.int/aux_poeorb/'. The script will assume that there are precise orbits if this folder is
        # defined, but falls back to other algorithms if needed.
        precise_orbits = dorisParameters.precise_orbits #'/......'
        polarisation = dorisParameters.polarisation

        # Here the doris inputfiles are stored. (Comes with the python scripts)
        input_files = dorisParameters.input_files  #'/......'


        profile.log_time_stamp('start')
        # Create a datastack using the stack function
        stack = StackData(track_dir=track_dir, shape_dat=shape_dat, start_date=start_date, end_date=end_date,
                          polarisation=polarisation, path=stack_path, db_type=2, precise_dir=precise_orbits)
        #profile.log_time_stamp('StackData')
        # Select the images which are new in this datastack.
        stack.select_image()
        # Then these images are unzipped
        stack.check_new_images(master=master_date)
        # All images which correspond with the start and end date are selected
        stack.unpack_image()
        # Based on the shape file bursts are selected for one date

        print('master date is ' + master_date)
        stack.select_burst(date=master_date)
        # And also for the other dates the needed bursts are selected
        stack.extend_burst()
        # Remove the images which are not fully present
        stack.remove_incomplete_images()
        # Now the exact coordinates of the different burst in the concatenated image is calculated
        stack.define_burst_coordinates(slaves=True)
        # Write the datastack to the stack_path folder
        stack.write_stack(write_path=stack_path,no_data=False)
        # A few auxiliary functions which are not strictly necessary.
        # Calculate the coverage of the different sub-swaths
        stack.swath_coverage()
        # Write the shapes from the bursts and swaths to a shapefile to check in a GIS program like Qgis.
        stack.write_shapes()
        profile.log_time_stamp('stack preparation finished')
        # Finally delete unzipped images
        stack.del_unpacked_image()

        import single_master_stack

        # Now we import the script to create a single master interferogram
        processing = SingleMaster(master_date=master_date, start_date=start_date,
                                                      end_date=end_date, stack_folder=stack_path,
                                                      input_files=input_files, processing_folder=stack_path)

        processing.remove_finished(step='dinsar')
        # Copy the necessary files to start processing
        profile.log_time_stamp('initialize')
        processing.initialize()

        # Calculate the coarse orbits of individual bursts
        if(dorisParameters.do_coarse_orbits):
            profile.log_time_stamp('coarse_orbits')
            processing.coarse_orbits()
        # Deramp the data of both slave and master
        if(dorisParameters.do_deramp):
            profile.log_time_stamp('deramp')
            processing.deramp(master=True) # Still needed for coherence...
        # Fake the use of fine window coregistration, which is officially not needed
        if(dorisParameters.do_fake_fine_coreg_bursts):
            profile.log_time_stamp('fake_fine_coreg_bursts')
            processing.fake_fine_coreg()
        # Perform DEM coregistration for individual bursts
        if(dorisParameters.do_dac_bursts):
            profile.log_time_stamp('dac_bursts')
            processing.dac_bursts()
        # Fake the use of coregmp, as the orbits are good enough for coregistration
        if(dorisParameters.do_fake_coreg_bursts):
            profile.log_time_stamp('fake_coreg_bursts')
            processing.fake_coregmp()
        # Resample individual bursts
        if(dorisParameters.do_resample):
            profile.log_time_stamp('resample')
            processing.resample()
        # Reramp burst
        if(dorisParameters.do_reramp):
            profile.log_time_stamp('reramp')
            processing.reramp()

        # Perform enhanced spectral diversity for full swath
        if(dorisParameters.do_esd):
            profile.log_time_stamp('esd')
            processing.esd()
        if (dorisParameters.do_network_esd):
            profile.log_time_stamp('network esd')
            processing.network_esd()
            
        # Make interferograms for individual bursts
        if(dorisParameters.do_interferogram):
            profile.log_time_stamp('interferogram')
            processing.interferogram()
        # Calculate earth reference phase from interferograms and combine for full swath
        if(dorisParameters.do_compref_phase):
            profile.log_time_stamp('compref_phase')
            processing.compref_phase()
        # Calculate height effects from interferograms and combine for full swath
        if(dorisParameters.do_compref_dem):
            profile.log_time_stamp('compref_dem')
            processing.compref_dem()
        # Remove earth reference phase from interferograms and combine for full swath
        if(dorisParameters.do_ref_phase):
            profile.log_time_stamp('ref_phase')
            processing.ref_phase(concatenate=False)
        # Remove height effects from interferograms and combine for full swath
        if(dorisParameters.do_ref_dem):
            profile.log_time_stamp('ref_dem')
            processing.ref_dem(concatenate=True, ras=True)
        # Geocode data
        if(dorisParameters.do_calc_coordinates):
            profile.log_time_stamp('calc_coordinates')
            processing.calc_coordinates()

        # Correct using ramp ifgs based on ESD
        if(dorisParameters.do_ESD_correct):
            profile.log_time_stamp('ESD_correct')
            processing.ESD_correct_ramp(filename='cint_srd.raw')
        # Compute coherence
        if(dorisParameters.do_coherence):
            profile.log_time_stamp('coherence')
            processing.coherence(ras=True)

        if(dorisParameters.do_phasefilt):
            profile.log_time_stamp('phasefilt')
            processing.phasefilt(ras=True)
        # Multilook filtered image and coherence image

        if(dorisParameters.do_multilooking):
            profile.log_time_stamp('multilooking')
            processing.multilook(step='coherence')
            processing.multilook(step='filtphase')
        # Unwrap image
        # processing.del_process('unwrap', type='ifgs', images=True)
        if(dorisParameters.do_unwrap):
            profile.log_time_stamp('unwrap')
            processing.unwrap()

        profile.log_time_stamp('end')

        print 'end sentinel 1 processing'




# This file contains several functions to read data from precise orbit files of sentinel-1 data.
# Scripts were created by  Wu Wenhao, Wuhan university and adapted by Gert Mulder, TU Delft

import time
import calendar
import numpy as np
import os, sys
from scipy.interpolate import interp1d
import scipy.interpolate as inter
import calendar


def orbit_read(input_EOF_FileName):

    try:
        import xml.etree.cElementTree as etree
    except:
        try:
            from lxml import etree
        except:
            #import xml.etree.ElementTree as etree
            print 'Failed to load lxml.etree or xml.etree.cElementTree'
            sys.exit(1)

    inTree = etree.parse(input_EOF_FileName)
    queryList = {
                # orbit inf
               'Mission'  : './/Earth_Explorer_Header/Fixed_Header/Mission',\
               'Validity_Start': './/Earth_Explorer_Header/Fixed_Header/Validity_Period/Validity_Start',\
               'Validity_Stop': './/Earth_Explorer_Header/Fixed_Header/Validity_Period/Validity_Stop',\
               'orbitABS' : './/Data_Block/List_of_OSVs/OSV/Absolute_Orbit',\
               'orbitTime': './/Data_Block/List_of_OSVs/OSV/UTC',\
               'orbitX'   : './/Data_Block/List_of_OSVs/OSV/X',\
               'orbitY'   : './/Data_Block/List_of_OSVs/OSV/Y',\
               'orbitZ'   : './/Data_Block/List_of_OSVs/OSV/Z',\
               }

    # temp variables and parameters
    container     = {}
    # containerTemp = {}
    events        = ('end',)

    for key in queryList.keys():

        try:
            vars()[key]
        except KeyError or NameError:
           vars()[key] = []

        for nodes in inTree.findall(queryList[key]):
            vars()[key].append(nodes.text)

        container[key] = vars()[key]
    return container

#--------------------------------------------------------
def interpolate_orbit(input_orbit_dir, date, input_orbit_type, input_interpolation_method, satellite='S1A'):

    orbit_time = calendar.timegm(time.strptime(date,'%Y-%m-%dT%H:%M:%S.%f'))
    date_start = calendar.timegm(time.strptime(date[:10],'%Y-%m-%d'))

    if input_orbit_type == 'POE':
        input_orbit_dir = os.path.join(input_orbit_dir, 'precise')
    elif input_orbit_type == 'RES':
        input_orbit_dir = os.path.join(input_orbit_dir, 'restituted')

    L = os.listdir(input_orbit_dir)
    Orbit_info = []

    if input_orbit_type == 'POE' and satellite == 'S1A':
        orbit_type = 'S1A_OPER_AUX_POEORB_OPOD_'
    elif input_orbit_type == 'RES' and satellite == 'S1A':
        orbit_type = 'S1A_OPER_AUX_RESORB_OPOD_'
    elif input_orbit_type == 'POE' and satellite == 'S1B':
        orbit_type = 'S1B_OPER_AUX_POEORB_OPOD_'
    elif input_orbit_type == 'RES' and satellite == 'S1B':
        orbit_type = 'S1B_OPER_AUX_RESORB_OPOD_'

    for d in L:
        if d.startswith(orbit_type):
            start_time = calendar.timegm(time.strptime(d[42:57], '%Y%m%dT%H%M%S'))
            end_time = calendar.timegm(time.strptime(d[58:73], '%Y%m%dT%H%M%S'))
            if (start_time < orbit_time) and (end_time > orbit_time):

                meta = orbit_read(os.path.join(input_orbit_dir, d))
                print(d)

                for i in range(len(meta['orbitTime'])):

                    point_time = calendar.timegm(time.strptime(meta['orbitTime'][i][4:-7], '%Y-%m-%dT%H:%M:%S'))
                    if (point_time > orbit_time-290) and (point_time < orbit_time+290):

                        Tuple_orbit=(float(hms2sec(meta['orbitTime'][i][4:].split('T')[1])),\
                                     float(meta['orbitX'][i]), float(meta['orbitY'][i]),\
                                     float(meta['orbitZ'][i]))
                        Orbit_info.append(Tuple_orbit)

    set_list=[]
    Orbit_info=sorted(Orbit_info)

    for element in range(len(Orbit_info)-1):
        temp_element     =Orbit_info[element][0]
        temp_element_add =Orbit_info[element+1][0]
        if int(temp_element) != int(temp_element_add):
            set_list.append(Orbit_info[element])

    Orbit_info=set_list

    orbit_Time=[]
    orbit_X   =[]
    orbit_Y   =[]
    orbit_Z   =[]

    for element in Orbit_info:
        orbit_Time.append(element[0])
        orbit_X.append(element[1])
        orbit_Y.append(element[2])
        orbit_Z.append(element[3])

    if len(orbit_X) == 0:
        return [], orbit_X, orbit_Y, orbit_Z

    del Orbit_info
    orbit_Time=np.array(orbit_Time)
    orbit_X   =np.array(orbit_X)
    orbit_Y   =np.array(orbit_Y)
    orbit_Z   =np.array(orbit_Z)
    if input_interpolation_method=='cubic':
        spl_x=interp1d(orbit_Time,orbit_X,kind='cubic')
        spl_y=interp1d(orbit_Time,orbit_Y,kind='cubic')
        spl_z=interp1d(orbit_Time,orbit_Z,kind='cubic')
    elif input_interpolation_method=='spline':
        spl_x = inter.InterpolatedUnivariateSpline (orbit_Time,orbit_X)
        spl_y = inter.InterpolatedUnivariateSpline (orbit_Time,orbit_Y)
        spl_z = inter.InterpolatedUnivariateSpline (orbit_Time,orbit_Z)

    input_time = np.arange((orbit_time - date_start) - 100, (orbit_time - date_start) + 100).astype(dtype='int32')
    out_orbit_X=spl_x(input_time)
    out_orbit_Y=spl_y(input_time)
    out_orbit_Z=spl_z(input_time)

    return input_time, out_orbit_X, out_orbit_Y, out_orbit_Z

#----------------------------------------------------------------
def hms2sec(hmsString,convertFlag='float'):
    # input hmsString syntax: XX:XX:XX.xxxxxx
    secString = int(hmsString[0:2])*3600 + \
        int(hmsString[3:5])*60 + \
        float(hmsString[6:])
    if convertFlag == 'int' :
        return int(secString)
    elif convertFlag == 'float' :
        return float(secString)
    else:
        return int(secString)


#!/usr/bin/env python
import os,sys,time

if __name__ == "__main__":
    # If calling script directly we have to load the package first to our python path
    folder = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
    print(folder)
    sys.path.extend([folder])

import resdata as resdata


def dump_data(input_file,res_file, output_file='', coordinates=[]):
    # This function dumps a .raw file from the original .tif sentinel data. The input_file is the .tif file and the
    # res_file is the .res file that corresponds with the output file. Coordinates is an optional variable which can be
    # called if the lines and pixels are not yet defined in the .res file.

    res_vars = resdata.ResData(filename=res_file)
    res_vars.res_read()

    # Check if information about crop is available

    if not coordinates:
        if res_vars.process_control['crop'] == '0':
            print 'There is no information available about how to crop this file!'
            return
        else:
            outputWinFirstPix = int(res_vars.processes['crop']['First_pixel (w.r.t. original_image)'])
            outputWinLastPix = int(res_vars.processes['crop']['Last_pixel (w.r.t. original_image)'])
            outputWinFirstLine = int(res_vars.processes['crop']['First_line (w.r.t. tiff_image)'])
            outputWinLastLine = int(res_vars.processes['crop']['Last_line (w.r.t. tiff_image)'])

    else:
        outputWinFirstPix = coordinates[0]
        outputWinLastPix = coordinates[1]
        outputWinFirstLine = coordinates[2]
        outputWinLastLine = coordinates[3]

    if not output_file:
        if res_vars.process_control['crop'] == '1':
            if 'Data_output_file' in res_vars.processes['crop'].keys():
                output_file = os.path.join(os.path.dirname(res_file), res_vars.processes['crop']['Data_output_file'])
        if not output_file:
            output_file = res_file.split(".")[0] + '.raw'

    # system parameters :: check whether its there

    gdalCall = 'gdal_translate'

    # Output data parameters
    outputDataFormat = 'MFF'
    outputDataType   = 'CInt16'

    cmd = '%s %s -ot %s -of %s %s' % (gdalCall,input_file,outputDataType,outputDataFormat,output_file)

    if outputWinFirstPix is not None:
        cmd = cmd + (' -srcwin %s %s %s %s' % (outputWinFirstPix,outputWinFirstLine,outputWinLastPix-outputWinFirstPix+1,outputWinLastLine-outputWinFirstLine+1))

    failure = os.system(cmd)
    if failure:
        print '%s: running %s failed' % (sys.argv[0],cmd)
        sys.exit(1)
    else:
        os.rename(os.path.splitext(output_file)[0]+'.j00',output_file)
        os.remove(os.path.splitext(output_file)[0]+'.hdr')
        os.remove(os.path.splitext(output_file)[0]+'.hdr.aux.xml')


# Actually execute the code to unzip one data file.
if __name__ == "__main__":

    input_file = sys.argv[1]
    res_file = sys.argv[2]
    output_file = sys.argv[3]

    dump_data(input_file, res_file, output_file, coordinates=[])





import math


def calculate_initial_compass_bearing(pointA, pointB):

    if (type(pointA) != tuple) or (type(pointB) != tuple):
        raise TypeError("Only tuples are supported as arguments")

    lat1 = math.radians(pointA[0])
    lat2 = math.radians(pointB[0])

    diffLong = math.radians(pointB[1] - pointA[1])

    x = math.sin(diffLong) * math.cos(lat2)
    y = math.cos(lat1) * math.sin(lat2) - (math.sin(lat1)
            * math.cos(lat2) * math.cos(diffLong))

    initial_bearing = math.atan2(x, y)

    # Now we have the initial bearing but math.atan2 return values
    # from -180 to + 180 which is not what we want for a compass bearing
    # The solution is to normalize the initial bearing as shown below
    initial_bearing = math.degrees(initial_bearing)
    compass_bearing = (initial_bearing + 360) % 360

    return compass_bearing


# This function performs an xml query on a provided xml file.

import sys
import collections


def xml_query(input_xml):

    try:
        import xml.etree.cElementTree as etree
    except:
        try:
            from lxml import etree
        except:
            #import xml.etree.ElementTree as etree
            print 'Failed to load lxml.etree or xml.etree.cElementTree'
            sys.exit(1)

    inTree = etree.parse(input_xml)

    queryList = collections.OrderedDict([
        ('Volume_file'                                  , 'dummy'),
        ('Volume_ID'                                    , './/adsHeader/missionDataTakeId'),
        ('Volume_identifier'                            , 'dummy'),
        ('Volume_set_identifier'                        , 'dummy'),
        ('Number of records in ref. file'               , 'dummy'),
        ('SAR_PROCESSOR'                                , 'update_1'),
        ('SWATH'                                        , './/adsHeader/swath'),
        ('PASS'                                         , './/generalAnnotation/productInformation/pass'),
        ('IMAGE_MODE'                                   , './/adsHeader/mode'),
        ('polarisation'                                 , './/adsHeader/polarisation'),
        ('Product type specifier'                       , './/adsHeader/missionId'),
        ('Logical volume generating facility'           , 'dummy'),
        ('Location and date/time of product creation'   , 'dummy'),
        ('Number_of_lines_Swath'                        , './/imageAnnotation/imageInformation/numberOfLines'),
        ('number_of_pixels_Swath'                       , './/imageAnnotation/imageInformation/numberOfSamples'),
        ('rangePixelSpacing'                            , './/imageAnnotation/imageInformation/rangePixelSpacing'),
        ('azimuthPixelSpacing'                          , './/imageAnnotation/imageInformation/azimuthPixelSpacing'),
        ('total_Burst'                                  , 'update_1'),
        ('Burst_number_index'                           , 'update_2'),
        ('RADAR_FREQUENCY (HZ)'                         , './/generalAnnotation/productInformation/radarFrequency'),
        ('Scene identification'                         , 'update_1'),
        ('Scene location'                               , 'update_1'),
        ('Sensor platform mission identifer'            , './/adsHeader/missionId'),
        ('Scene_center_heading'                         , './/generalAnnotation/productInformation/platformHeading'),
        ('Scene_centre_latitude'                        , 'update_2'),
        ('Scene_centre_longitude'                       , 'update_2'),
        ('Radar_wavelength (m)'                         , 'update_1'),
        ('Azimuth_steering_rate (deg/s)'                , './/generalAnnotation/productInformation/azimuthSteeringRate'),
        ('Pulse_Repetition_Frequency_raw_data(TOPSAR)'  , './/generalAnnotation/downlinkInformationList/downlinkInformation/prf'),
        ('First_pixel_azimuth_time (UTC)'               , 'update_2'),
        ('Pulse_Repetition_Frequency (computed, Hz)'    , './/imageAnnotation/imageInformation/azimuthFrequency'),
        ('Azimuth_time_interval (s)'                    , './/imageAnnotation/imageInformation/azimuthTimeInterval'),
        ('Total_azimuth_band_width (Hz)'                , './/imageAnnotation/processingInformation/swathProcParamsList/swathProcParams/azimuthProcessing/totalBandwidth'),
        ('Weighting_azimuth'                           , './/imageAnnotation/processingInformation/swathProcParamsList/swathProcParams/azimuthProcessing/windowType'),
        ('Range_time_to_first_pixel (2way) (ms)'        , 'update_1'),
        ('Range_sampling_rate (computed, MHz)'          , 'update_1'),
        ('Total_range_band_width (MHz)'                 , 'update_1'),
        ('Weighting_range'                              , './/imageAnnotation/processingInformation/swathProcParamsList/swathProcParams/rangeProcessing/windowType'),
        ('DC_reference_azimuth_time'                    , 'update_2'),
        ('DC_reference_range_time'                      , 'update_2'),
        ('Xtrack_f_DC_constant (Hz, early edge)'        , 'update_2'),
        ('Xtrack_f_DC_linear (Hz/s, early edge)'        , 'update_2'),
        ('Xtrack_f_DC_quadratic (Hz/s/s, early edge)'   , 'update_2'),
        ('FM_reference_azimuth_time'                    , 'update_2'),
        ('FM_reference_range_time'                      , 'update_2'),
        ('FM_polynomial_constant_coeff (Hz, early edge)', 'update_2'),
        ('FM_polynomial_linear_coeff (Hz/s, early edge)', 'update_2'),
        ('FM_polynomial_quadratic_coeff (Hz/s/s, early edge)', 'update_2'),
        ('Datafile'                                     , 'update_2'),
        ('Dataformat'                                   , 'update_2'),
        ('Number_of_lines_original'                     , 'update_2'),
        ('Number_of_pixels_original'                    , 'update_2')
    ])

    queryList_aux = collections.OrderedDict([
        ('Swath_startTime'                      , './/adsHeader/startTime'),
        ('Swath_stopTime'                       , './/adsHeader/stopTime'),
        ('imageLines'                           , './/swathTiming/linesPerBurst'),
        ('imagePixels'                          , './/swathTiming/samplesPerBurst'),
        ('firstValidSample'                     , './/swathTiming/burstList/burst/firstValidSample'),
        ('lastValidSample'                      , './/swathTiming/burstList/burst/lastValidSample'),
        ('productSpec'                          , './/generalHeader/referenceDocument'),
        ('productVolDate'                       , './/setup//IOCSAuxProductGenerationTimeUTC'),
        ('productDate'                          , './/generalHeader/generationTime'),
        ('productFacility'                      , './/productInfo/generationInfo/level1ProcessingFacility'),
        ('scenePol'                             , './/adsHeader/polarisation'),
        ('sceneMode'                            , './/adsHeader/mode'),
        ('sceneCenLine_number'                  , './/geolocationGrid/geolocationGridPointList/geolocationGridPoint/line'),
        ('sceneCenPixel_number'                 , './/geolocationGrid/geolocationGridPointList/geolocationGridPoint/pixel'),
        ('sceneCenLat'                          , './/geolocationGrid/geolocationGridPointList/geolocationGridPoint/latitude'),
        ('sceneCenLon'                          , './/geolocationGrid/geolocationGridPointList/geolocationGridPoint/longitude'),
        ('height'                               , './/geolocationGrid/geolocationGridPointList/geolocationGridPoint/height'),
        ('azimuthTime'                          , './/geolocationGrid/geolocationGridPointList/geolocationGridPoint/azimuthTime'),
        ('sceneRecords'                         , './/imageDataInfo/imageRaster/numberOfRows'),
        ('orbitABS'                             , './/adsHeader/absoluteOrbitNumber'),
        ('orbitTime'                            , './/generalAnnotation/orbitList/orbit/time'),
        ('orbitX'                               , './/generalAnnotation/orbitList/orbit/position/x'),
        ('orbitY'                               , './/generalAnnotation/orbitList/orbit/position/y'),
        ('orbitZ'                               , './/generalAnnotation/orbitList/orbit/position/z'),
        ('rangeRSR'                             , './/generalAnnotation/productInformation/rangeSamplingRate'),
        ('rangeBW'                              , './/imageAnnotation/processingInformation/swathProcParamsList/swathProcParams/rangeProcessing/processingBandwidth'),
        ('rangeTimePix'                         , './/imageAnnotation/imageInformation/slantRangeTime'),
        ('azimuthTimeStart'                     , './/swathTiming/burstList/burst/azimuthTime'),
        ('heading'                              , './/generalAnnotation/productInformation/platformHeading'),
        ('doppler_azimuth_Time'                 , './/dopplerCentroid/dcEstimateList/dcEstimate/azimuthTime'),
        ('doppler_range_Time'                   , './/dopplerCentroid/dcEstimateList/dcEstimate/t0'),
        ('dopplerCoeff'                         , './/dopplerCentroid/dcEstimateList/dcEstimate/dataDcPolynomial'),
        ('azimuthFmRate_reference_Azimuth_time' , './/generalAnnotation/azimuthFmRateList/azimuthFmRate/azimuthTime'),
        ('azimuthFmRate_reference_Range_time'   , './/generalAnnotation/azimuthFmRateList/azimuthFmRate/t0'),
        ('azimuthFmRate_c0'                     , './/generalAnnotation/azimuthFmRateList/azimuthFmRate/c0'),
        ('azimuthFmRate_c1'                     , './/generalAnnotation/azimuthFmRateList/azimuthFmRate/c1'),
        ('azimuthFmRate_c2'                     , './/generalAnnotation/azimuthFmRateList/azimuthFmRate/c2'),
        ('azimuthFmRatePolynomial'              , './/generalAnnotation/azimuthFmRateList/azimuthFmRate/azimuthFmRatePolynomial')

    ])

    # Now find the variables of queryList and queryList_aux in the xml data.

    for key in queryList.keys():
        try:
            vars()[key]
        except KeyError or NameError:
            vars()[key] = []

        if queryList[key][0:3] == './/':
            for nodes in inTree.findall(queryList[key]):
                vars()[key].append(nodes.text)
            dat = vars()[key]
            if isinstance(dat,list):
                dat = dat[0]
            if isinstance(dat,(int,float)):
                dat = str(dat)
            queryList[key] = dat

    for key in queryList_aux.keys():
        try:
            vars()[key]
        except KeyError or NameError:
            vars()[key] = []

        for nodes in inTree.findall(queryList_aux[key]):
            vars()[key].append(nodes.text)
        queryList_aux[key] = vars()[key]

    # Finally do the first update
    queryList['SAR_PROCESSOR'] = 'Sentinel-' + queryList['Sensor platform mission identifer'][-2:]
    queryList['total_Burst'] = str(len(queryList_aux['azimuthTimeStart']))
    queryList['Scene identification'] = 'Orbit: '+ queryList_aux['orbitABS'][0]
    queryList['Scene location'] = 'lat: ' + queryList_aux['sceneCenLat'][0] + ' lon:' + queryList_aux['sceneCenLon'][0]
    queryList['Radar_wavelength (m)'] = "{:.9f}".format(299792458.0/float(queryList['RADAR_FREQUENCY (HZ)']))
    queryList['Range_time_to_first_pixel (2way) (ms)'] = "{:.15f}".format(float(queryList_aux['rangeTimePix'][0])*1000)
    queryList['Range_sampling_rate (computed, MHz)'] = "{:.9f}".format(float(queryList_aux['rangeRSR'][0])/1000000)
    queryList['Total_range_band_width (MHz)'] = "{:.9f}".format(float(queryList_aux['rangeBW'][0])/1000000)

    deldict = ['orbitABS','rangeBW']
    for d in deldict:
        queryList_aux.pop(d)

    queryList['aux'] = queryList_aux

    return queryList



#!/usr/bin/env python
import os,sys,time

def usage():
    print '\nUsage: python sec2hhmmss.py time'
    print '  where time is the time of day in seconds.'
    print ' '
    print ' Example '
    print ' sec2hhmmss.py 59800.445398'
    print ' 16:36:40.393'

try:
    timeOfDay  = sys.argv[1] 
except:
    print 'Unrecognized input'
    usage()
    sys.exit(1)

timeOfDay  = float(sys.argv[1])

hh = timeOfDay//3600
timeOfDay = timeOfDay%3600
mm = timeOfDay//60
ss = timeOfDay%60
print "%d:%d:%f" %(hh,mm,ss)



# Function created by Gert Mulder
# Institute TU Delft
# Date 9-11-2016
# Part of Doris 5.0

# This function makes a setup of the processing folder, based on a single shapefile.
# Inputs are
# - the satellite sensor used
# - the shapefile
# - the processing folder we want to create
# - the dem source folder where the intermediate DEM data is stored

import os
from create_dem import CreateDem
from create_inputfiles import CreateInputFiles
from create_doris_input_xml import CreateDorisInputXml
from create_datastack_bash import CreateBash
import xml.etree.ElementTree as ET

class PrepareDatastack(object):

    def __init__(self):
        return

    def prepare(self, inputfile):

        # This will first create the framework with data folders the stackfolder should contain
        # a dorisparameters file.

        doris_input_xml = CreateDorisInputXml(inputfile)

        folders = ['input_files', 'stack', 'dem']
        for foldername in folders:
            if not os.path.exists(os.path.join(doris_input_xml.get_value('datastack_folder'), foldername)):
                os.mkdir(os.path.join(doris_input_xml.get_value('datastack_folder'), foldername))

        # Then create the dem file
        password_file = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))),'install', 'doris_config.xml')
        tree_doris = ET.parse(password_file)
        settings_doris = tree_doris.getroot()
        srtm_username = settings_doris.find('.usgs_username').text
        srtm_password = settings_doris.find('.usgs_password').text
        dem_out = os.path.join(doris_input_xml.get_value('dem_folder'), 'dem.raw')
        dem_var = dem_out + '.var'

        dem = CreateDem()
        if doris_input_xml.get_value('generate_dem'.lower())=='yes':
            dem.create(doris_input_xml.get_value('shape_file_path'), dem_out, dem_var, resample=None,
                                    doris_input=True, rounding=0.1, border=1.5,
                                    data_folder=doris_input_xml.get_value('dem_processing_folder'), quality='SRTM3',
                                    password=srtm_password, username=srtm_username)

        ## Then create the inputfiles
        inputfiles_folder = os.path.join(doris_input_xml.get_value('datastack_folder'), 'input_files')
        xml_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'inputfile_template.xml')
        satellite = 'sentinel-1'
        CreateInputFiles(dem_var, xml_file, satellite).create(inputfiles_folder)

        root_folder = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        CreateBash().create(doris_input_xml.get_value('datastack_folder'), root_folder, doris_input_xml.get_value('cores'))

